/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: Bezahlung.java,v 1.4 2004/02/10 16:20:32 tk4 Exp $
 */
package haendler;

import idman.mngt.Context;

import java.io.File;
import java.io.FileWriter;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Klasse zur Verarbeitung von Bezahldokumenten.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: Bezahlung.java,v 1.4 2004/02/10 16:20:32 tk4 Exp $
 */
public class Bezahlung {

	private boolean Status = true;
	private String ErrorMsg = "";
	private BezahlSatz neueBezahlung = new BezahlSatz();
	private Context context = null;

	public Bezahlung(
		Document doc,
		Haendler haendlerInstance,
		Context context) {
		this.context = context;
		int index = -1;
		readData(doc, haendlerInstance);
		if (Status == true) {
			index = checkBestellung(haendlerInstance);
		}
		// wenn alles OK --> Bestellung lschen
		if (Status == true) {
			haendlerInstance.removeBestellSatz(index);
		}
	}
	protected void readData(Document doc, Haendler haendlerInstance) {

		// Erstmal testen...
		Document[] docs = haendlerInstance.verify(doc, context);
		if ((docs == null) || (docs.length != 1)) {
			Status = false;
			if (docs.length == 0) {
				ErrorMsg = "Bezahlung nicht signiert.";
			} else {
				ErrorMsg = "Bezahlung verletzt Protokoll.";
			}
			return;
		}
		doc = docs[0];

		//Treuhaender-Daten auslesen
		NodeList treuhaenderList = doc.getElementsByTagName("Treuhaender");
		if (treuhaenderList.getLength() == 0) {
			ErrorMsg = "Bezahlung ... kein Treuhaender gefunden";
			Status = false;
			return;
		}
		Node thNode = treuhaenderList.item(0);
		String TName = "";
		String TAdresse = "";
		String TTelefon = "";
		String TIPAdresse = "";
		String TPort = "";
		if (thNode.hasChildNodes()) {
			thNode = thNode.getFirstChild();
			do {
				if (thNode.getNodeName().equals("TName")) {
					TName = thNode.getFirstChild().getNodeValue();
				}
				if (thNode.getNodeName().equals("TAdresse")) {
					TAdresse = thNode.getFirstChild().getNodeValue();
				}
				if (thNode.getNodeName().equals("TTelefon")) {
					TTelefon = thNode.getFirstChild().getNodeValue();
				}
				if (thNode.getNodeName().equals("TIPAdresse")) {
					TIPAdresse = thNode.getFirstChild().getNodeValue();
				}
				if (thNode.getNodeName().equals("TPort")) {
					TPort = thNode.getFirstChild().getNodeValue();
				}
				thNode = thNode.getNextSibling();
			} while (thNode != null);
		}
		neueBezahlung.setTreuhaender(
			TName,
			TAdresse,
			TTelefon,
			TIPAdresse,
			TPort);
		//	System.out.println("Bezahlung ... Treuhaender-Pseudonym gefunden: "+ neueBezahlung.getTName());

		//Haendler-Daten auslesen und vergleichen
		NodeList haendlerList = doc.getElementsByTagName("Haendler");
		if (haendlerList.getLength() == 0) {
			ErrorMsg = "Bezahlung ... kein Haendler gefunden";
			Status = false;
			return;
		}
		Node haendlerNode = haendlerList.item(0);
		String HName = "";
		String HIPAdresse = "";
		String HPort = "";
		if (haendlerNode.hasChildNodes()) {
			haendlerNode = haendlerNode.getFirstChild();
			do {
				if (haendlerNode.getNodeName().equals("HName")) {
					HName = haendlerNode.getFirstChild().getNodeValue();
				}
				if (haendlerNode.getNodeName().equals("HIPAdresse")) {
					HIPAdresse = haendlerNode.getFirstChild().getNodeValue();
				}
				if (haendlerNode.getNodeName().equals("HPort")) {
					HPort = haendlerNode.getFirstChild().getNodeValue();
				}
				haendlerNode = haendlerNode.getNextSibling();
			} while (haendlerNode != null);
		}
		neueBezahlung.setHaendler(HName, HIPAdresse, HPort);
		//	System.out.println("Bezahlung ... Haendler-Pseudonym gefunden: "+ neueBezahlung.getHName());
		/* Das war einmal
		if (!haendlerInstance.verify(doc, neueBezahlung.getTName())) {
		    System.out.println("Bezahlung: Signatur FALSCH ...");
		    ErrorMsg = "Bezahlung ... Validierung des Treuhaender-Pseudonym fehlgeschlagen";
		    Status=false;
		    return;
		}
		*/

		//Bestell_ID auslesen
		NodeList bestellIDList = doc.getElementsByTagName("Bestell_ID");
		if (bestellIDList.getLength() == 0) {
			ErrorMsg = "Bezahlung ... Keine Bestell_ID angegeben";
			Status = false;
			return;
		} else {
			neueBezahlung.setBestellID(
				bestellIDList.item(0).getFirstChild().getNodeValue());
			//	   System.out.println("Bezahlung ... Bestell_ID: " + neueBezahlung.getBestellID());
		}

		//Ges_Preis auslesen
		NodeList PreisList = doc.getElementsByTagName("Ges_Preis");
		if (PreisList.getLength() == 0) {
			ErrorMsg = "Bezahlung ... Keine Ges_Preis angegeben";
			Status = false;
			return;
		} else {
			String gesPreis =
				parsePrice(PreisList.item(0).getFirstChild().getNodeValue());
			neueBezahlung.setGesPreis(gesPreis);
			//	   System.out.println("Bezahlung ... GesPreis: " + neueBezahlung.getGesPreis());
		}

		//Geld auslesen
		NodeList GeldList = doc.getElementsByTagName("Geld");
		if (GeldList.getLength() == 0) {
			ErrorMsg = "Bezahlung ... Keine Geld enthalten";
			Status = false;
			return;
		} else {
			String Geld = GeldList.item(0).getFirstChild().getNodeValue();
			neueBezahlung.setGeld(Geld);
			//	   System.out.println("Bezahlung ... Geld: " + neueBezahlung.getGeld());
		}

	} // END readData(Document doc, Haendler haendlerInstance)

	public boolean getStatus() {
		return Status;
	}

	public String getError() {
		return ErrorMsg;
	}

	public Document getBezahlung() {
		return null;
	}

	protected static String parsePrice(String price) {
		price = price.replace(',', '.');
		int i = price.indexOf("DM");
		if (i == -1)
			return price;
		int l = price.length();
		return price.substring(0, i) + price.substring(i + 2, l);
	}

	protected int checkBestellung(Haendler haendlerInstance) {
		Vector b = haendlerInstance.getBestellList();
		if (b.isEmpty()) {
			Status = false;
			ErrorMsg = "Bezahlung ... Es liegen keine Bestellungen vor";
			return -1;
		}
		Bestellung.BestellSatz bestellung = null;
		String BestellID = "";
		int i;
		for (i = 0; i < b.size(); i++) {
			bestellung = (Bestellung.BestellSatz) b.get(i);
			BestellID = bestellung.getBestellID();
			System.out.println("Bezahlung: " + BestellID);
			if (BestellID.equals(neueBezahlung.getBestellID())) {
				break;
			}
		}
		// Ges_Preise vergleichen ...
		if (bestellung.getGesPreis().equals(neueBezahlung.getGesPreis())) {
			System.out.println("Bezahlung: Gesamtpreis stimmt");
			// Virtuelles Geld sichern
			try {
				File outFile =
					new File(neueBezahlung.getBestellID() + ".money");
				outFile.delete();
				outFile.createNewFile();
				FileWriter fw = new FileWriter(outFile);
				fw.write(neueBezahlung.getGeld());
				fw.close();
			} catch (Exception ex) {
				ErrorMsg = "Bezahlung ... Geld speichern fehlgeschlagen";
				Status = false;
			}
			return i;
		} else {
			Status = false;
			ErrorMsg =
				"Bezahlung ... Gesamtpreise von Bestellung und Bezahlung stimmen nicht ueberein";
			return -1;
		}
	}

	public static class BezahlSatz {

		private String TName;
		private String TAdresse;
		private String TTelefon;
		private String TIPAdresse;
		private String TPort;
		private String HName;
		private String HIPAdresse;
		private String HPort;
		private String BestellID;
		private String GesPreis;
		private String Geld;

		public BezahlSatz() {
		}
		protected void setBestellID(String BID) {
			BestellID = BID;
		}
		protected String getBestellID() {
			return BestellID;
		}
		protected void setTreuhaender(
			String TN,
			String TAdr,
			String TTel,
			String TIP,
			String TP) {
			TName = TN;
			TAdresse = TAdr;
			TTelefon = TTel;
			TIPAdresse = TIP;
			TPort = TP;
		}
		protected void setTName(String TN) {
			TName = TN;
		}
		protected void setTAdresse(String TAdr) {
			TAdresse = TAdr;
		}
		protected void setTTelefon(String TTel) {
			TTelefon = TTel;
		}
		protected void setTIPAdresse(String TIP) {
			TIPAdresse = TIP;
		}
		protected void setTPort(String TP) {
			TPort = TP;
		}
		protected String getTName() {
			return TName;
		}
		protected String getTAdresse() {
			return TAdresse;
		}
		protected String getTTelefon() {
			return TTelefon;
		}
		protected String getTIPAdresse() {
			return TIPAdresse;
		}
		protected String getTPort() {
			return TPort;
		}
		protected void setHaendler(String HN, String HIP, String HP) {
			HName = HN;
			HIPAdresse = HIP;
			HPort = HP;
		}
		protected void setHName(String HN) {
			HName = HN;
		}
		protected String getHName() {
			return HName;
		}
		protected void setHIPAdresse(String HIP) {
			HIPAdresse = HIP;
		}
		protected void setHPort(String HP) {
			HPort = HP;
		}
		protected String getHIPAdresse() {
			return HIPAdresse;
		}
		protected String getHPort() {
			return HPort;
		}
		protected void setGesPreis(String GP) {
			GesPreis = GP;
		}
		protected String getGesPreis() {
			return GesPreis;
		}
		protected void setGeld(String G) {
			Geld = G;
		}
		protected String getGeld() {
			return Geld;
		}
	}
}
