/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: ClientSalesMan.java,v 1.10 2004/02/12 08:12:41 tk4 Exp $
 */
package haendler;

import idman.mngt.Context;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ssonet.net.ActionConfiguration;
import org.ssonet.net.ConnectionListener;
import org.ssonet.net.SSONETContext;
import org.ssonet.net.impl.SSONETServerClientSocket;
import org.w3c.dom.Document;

/**
 * Kapselt die Kommunikation via Sockets mit dem Client.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: ClientSalesMan.java,v 1.10 2004/02/12 08:12:41 tk4 Exp $
 */
public class ClientSalesMan extends Thread implements ConnectionListener {

	/* (non-Javadoc)
	 * @see org.ssonet.net.ConnectionListener#connectionStateChanged(org.ssonet.net.SSONETContext, int)
	 */
	public synchronized void connectionStateChanged(
		SSONETContext context,
		int currentState) {
		if (currentState == RUNNING_STATE) {
			notifyAll();
		}
	}
	public synchronized void waitFor(SSONETContext context) {
		context.addConnectionListener(this);
		if (context.getConnectionState() != RUNNING_STATE) {
			try {
				wait();
			} catch (Exception e) {
			}
		}
		context.removeConnectionListener(this);
	}
	
	private SSONETServerClientSocket s = null;
	private Haendler haendler = null;
	private Context context = null;

	protected static DocumentBuilderFactory dbf = null;
	protected static DocumentBuilder db = null;
	protected static TransformerFactory tf = null;
	protected static Transformer t = null;

	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
			tf = TransformerFactory.newInstance();
			t = tf.newTransformer();
		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		} catch (TransformerConfigurationException tce) {
			tce.printStackTrace();
		}
	}
	public ClientSalesMan(SSONETServerClientSocket s, Haendler haendler) {
		this.s = s;
		this.haendler = haendler;
	}

	private Document performRequest(InputStream is) {
		try {
			int i, len = 0;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			if (len == 0)
				return null;

			byte[] tmp = new byte[len];
			for (int c = 0, j = 0;
				(c < tmp.length)
					&& (-1 != (j = is.read(tmp, c, tmp.length - c)));
				c += j);
			System.out.write(tmp);
			ByteArrayInputStream bais = new ByteArrayInputStream(tmp);

			Document request = db.parse(bais);
			Document response = null;
			response = haendler.handleRequest(request, context);
			return response;
		} catch (Exception e) {
			System.out.println(
				"ClientSalesMan#performRequest: " + e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
	public void connect() {
		try {
			InputStream is = s.getInputStream();
			OutputStream os = s.getOutputStream();
			this.context = (Context) s.getContext();
			waitFor(context);
			context.setTargetAddress(s.getInetAddress());
			context.setTargetPort(s.getPort());
			System.out.println("ClientSalesMan#connect: " + s.toString());
			Document response = performRequest(is);
			if (response != null) {
				//   System.out.println("Einlesen");
				sendResponse(response, os);
			} else {
				os.write(new byte[4]);
			} //else
			os.flush();
			os.close();
			is.close();
			s.close();
		} catch (IOException ioe) {
			System.out.println("ClientSalesMan#connect: " + ioe.getMessage());
		}
	}

	// zurckschreiben der Antwort bei erfolgreichem Test
	private void sendResponse(Document response, OutputStream os) {
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			Transformer t = tf.newTransformer();
			t.transform(new DOMSource(response), new StreamResult(baos));

			byte[] tmp = baos.toByteArray();
			baos.close();

			byte[] len =
				{
					(byte) (tmp.length >> 24 & 255),
					(byte) (tmp.length >> 16 & 255),
					(byte) (tmp.length >> 8 & 255),
					(byte) (tmp.length & 255)};
			os.write(len);
			os.write(tmp);
			os.flush();
		} catch (IOException ioe) {
			System.out.println(
				"ClientSalesMan#makeResponse: " + ioe.getMessage());
		} catch (TransformerConfigurationException tce) {
			System.out.println(
				"ClientSalesMan#makeResponse: " + tce.getMessage());
		} catch (TransformerException te) {
			System.out.println(
				"ClientSalesMan#makeResponse: " + te.getMessage());
		}
	}
	public void run() {
		try {
			connect();
		} catch (Exception ie) {
			System.out.println(
				ie.toString() + "   Fehler: Verbindung abgebrochen");
			ie.printStackTrace();
		}
	}
	/* (non-Javadoc)
	 * @see org.ssonet.net.ConnectionListener#negotiationRequestReceived(org.ssonet.net.SSONETContext, int)
	 */
	public void negotiationRequestReceived(SSONETContext context, int negotiationType) {
	}
	/* (non-Javadoc)
	 * @see org.ssonet.net.ConnectionListener#decideSecurityGoal(int, org.ssonet.net.ActionConfiguration, org.ssonet.net.SSONETContext)
	 */
	public boolean decideSecurityGoal(int securityGoal, ActionConfiguration ownConfiguration, SSONETContext context) {
		return false;
	}

}
