/**
 * @version working
 * @author Torsten Keil (lehrprojekt)
 * @author Thomas Kriegelstein
 * $Id: Haendler.java,v 1.12 2004/02/10 16:20:32 tk4 Exp $
 */
package haendler;

import idman.gui.LocalManagementGUI;
import idman.mngt.Context;
import idman.mngt.Management;
import idman.rules.ChainEvaluationListener;
import idman.rules.ChainEvaluator;
import idman.rules.Condition;
import idman.rules.DefaultCertificateListener;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.WindowConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ssonet.net.ActionConfiguration;
import org.ssonet.net.CertificateListener;
import org.ssonet.net.SSONETContext;
import org.ssonet.net.SSONETContextFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * Zentrale Bearbeitungsstelle im Hndler.
 * @author Torsten Keil (lehrprojekt)
 * @author Thomas Kriegelstein
 * @version $Id: Haendler.java,v 1.12 2004/02/10 16:20:32 tk4 Exp $
 */
public class Haendler
	implements ChainEvaluationListener, SSONETContextFactory {

	private Properties settings = null;
	private Management manager = null;
	private ServerSalesMan server = null;
	private CertificateListener listener = null;
	private LocalManagementGUI ui = null;

	private String HName_ = "";
	private String HIPAdresse_ = "";
	private String HPort_ = "";
	private Vector bestellList = new Vector();
	private Vector mangelList = new Vector();
	private static DocumentBuilderFactory dbf = null;
	private static DocumentBuilder db = null;
	private static MessageDigest digest = null;

	/** Initialisierung der XML-Verarbeitung und des Digest-Algorithmus. */
	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setIgnoringComments(false);
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
		} catch (Exception e) {
			System.err.println(
				"Haendler<init>: Could not create Factory: " + e.getMessage());
			e.printStackTrace();
		}
		try {
			digest = MessageDigest.getInstance("SHA1");
		} catch (Exception e) {
			System.err.println(
				"Haendler<init>: Could not create digest: " + e.getMessage());
			e.printStackTrace();
		}
	}

	/** Schreibt das Auftreten nach stdout und liefert <tt>true</tt>. */
	public boolean doWarning(
		String chainname,
		Condition cond,
		String id,
		byte[] content,
		Context context,
		String duno) {
		System.out.println("Warning in ChainEvaluator.");
		return true;
	}
	public SSONETContext createContext() {
		Context result = manager.createContext(HName_);
		// nur im Server... Da der den Partner am Anfang eh nicht kennt.
		result.setOwnActionConfiguration(new ActionConfiguration(3, 3, 3, 3));
		ChainEvaluator.evaluate(
			"Pseudonymverwaltung",
			this,
			null,
			null,
			result);
		result.addCertificateListener(listener);
		System.out.println("Context created.");
		return result;
	}

	public Vector getMangelList() {
		return mangelList;
	}
	public Vector getBestellList() {
		return bestellList;
	}
	public String getName() {
		return HName_;
	}
	public String getPort() {
		return HPort_;
	}
	public String getIPAdresse() {
		return HIPAdresse_;
	}
	public Haendler(Properties settings, String name, String port) {
		this.settings = settings;
		this.HName_ = name;
		this.HPort_ = port;
		try {
			InetAddress ia = InetAddress.getLocalHost();
			HIPAdresse_ = ia.getHostAddress();
		} catch (UnknownHostException uhe) {
			uhe.printStackTrace();
		}
		System.out.println(
			"Haendler: " + HName_ + "/" + HIPAdresse_ + ":" + HPort_);
		int p = 0;
		try {
			p = Integer.parseInt(port);
		} catch (NumberFormatException nfe) {
			// should not happen!
			nfe.printStackTrace();
		}
		this.manager = new idman.mngt.impl.LocalManagement(settings);
		this.manager.setStatus("Starte Server");
		this.server = new ServerSalesMan(this, p, this);
		this.listener = new DefaultCertificateListener(this);
		try {
			this.manager.setStatus("ffne GUI");
			JFrame frame = new JFrame(HName_);
			this.ui = new LocalManagementGUI(manager, frame, settings);
			frame.getContentPane().add(ui);
			frame.setSize(400, 400);
			frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			frame.show();
		} catch (Error e) {
			System.err.println("Could not open GUI: " + e.getMessage());
		}
	}
	public void start() {
		System.out.println("Haendler: gestartet auf: " + HPort_);
		manager.setStatus("Hndler bereit.");
		server.start();
	}
	public void addBestellSatz(Bestellung.BestellSatz bs) {
		bestellList.add(bs);
	}
	public void removeBestellSatz(int index) {
		bestellList.remove(index);
	}
	public void addMangelEintrag(Mangel.MangelEintrag me) {
		mangelList.add(me);
	}
	public void removeMangelEintrag(int index) {
		mangelList.remove(index);
	}
	public Document[] verify(Document d, Context c) {
		if (d == null)
			return new Document[0];
		return manager.verifyDocument(d, c);
	}
	public Document sign(Document d, Context c) {
		if (d == null)
			return null;
		return manager.signDocument(d, c);
	}
	public String hash(String s) {
		String result = null;
		byte[] tmp = s.getBytes();
		tmp = digest.digest(tmp);
		String hash = new String(tmp);
		result = "" + hash.hashCode();
		return result;
	}
	public Document getDocFromFile(String filename) {
		try {
			File file = new File(filename);
			if (file.exists())
				return getDocFromFile(new FileInputStream(file));
			else
				return getDocFromFile(
					getClass().getResourceAsStream("/" + filename));
		} catch (Exception e) {
			System.out.println("Fehler in getDocFromFile" + e);
		}
		return null;
	}
	public Document getDocFromFile(InputStream stream) {
		try {
			Document doc = db.parse(stream);
			return doc;
		} catch (Exception e) {
			System.out.println("Fehler in getDocFromFile" + e);
		}
		return null; // Im Fehlerfall null zurueckliefern
	}
	/**
	 *  Sobald eine Verbindung aufgebaut und das Document
	 *  geparst wurde, wird diese Methode aufgerufen.
	 *
	 *  @param  request Request to handle
	 *  @param  context Information about the connection
	 *  @return Document response
	 */
	public Document handleRequest(Document request, Context context) {
		Document result = null;

		manager.registerContext(context);
		NodeList katalogList =
			request.getElementsByTagName("Kataloganforderung");
		NodeList bestellList = request.getElementsByTagName("Bestellungth");
		NodeList bezahlList = request.getElementsByTagName("Bezahlung");
		NodeList mangelList = request.getElementsByTagName("Mangelhaendler");

		if (katalogList.getLength() != 0) {
			// Entweder eine Kataloganforderung
			System.out.println("Kataloganforderung");
			Katalog katalog = new Katalog(request, this, context);
			if (!(true == katalog.getStatus())) {
				System.out.println(katalog.getError());
				result = null;
			} else {
				result = katalog.getKatalog();
			}
		} else if (bestellList.getLength() != 0) {
			// Oder eine Bestellung ...
			System.out.println("Bestellung");
			Bestellung bestellung = new Bestellung(request, this, context);
			if (!(true == bestellung.getStatus())) {
				System.out.println(bestellung.getError());
				result = null;
			} else {
				result = bestellung.getBestellung();
			}
		} else if (bezahlList.getLength() != 0) {
			// Oder eine Bezahlung ...
			System.out.println("Bezahlung");
			Bezahlung bezahlung = new Bezahlung(request, this, context);
			if (!(true == bezahlung.getStatus())) {
				System.out.println(bezahlung.getError());
				result = null;
			} else {
				result = bezahlung.getBezahlung();
			}
		} else if (mangelList.getLength() != 0) {
			// Oder ein Mangel ...
			System.out.println("Mangel");
			Mangel mangel = new Mangel(request, this, context);
			if (!(true == mangel.getStatus())) {
				System.out.println(mangel.getError());
				result = null;
			} else {
				result = mangel.getMangel();
			}
		}
		manager.unregisterContext(context);
		return result;
	}
}
