/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: Mangel.java,v 1.4 2004/02/10 16:20:32 tk4 Exp $
 */
package haendler;

import idman.mngt.Context;

import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Klasse zur Verarbeitung von Mangeldokumenten.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: Mangel.java,v 1.4 2004/02/10 16:20:32 tk4 Exp $
 */
public class Mangel {

	private boolean Status = true;
	private String ErrorMsg = "";
	private MangelSatz neuerMangel = new MangelSatz();
	private Bestellung.BestellSatz mangelBestellung = null;
	private Document mangel = null;
	private Context context = null;

	public boolean getStatus() {
		return Status;
	}
	public String getError() {
		return ErrorMsg;
	}
	public Document getMangel() {
		return mangel;
	}
	public Mangel(Document doc, Haendler haendlerInstance, Context context) {
		this.context = context;
		readData(doc, haendlerInstance);
		if (Status == true) {
			checkBestellung(haendlerInstance);
		}
		if (Status == true) {
			checkMangel(haendlerInstance);
		}
		if (Status == true) {
			repeatBestellung(haendlerInstance);
		}
	}
	protected String parsePrice(String price) {
		price = price.replace(',', '.');
		int i = price.indexOf("DM");
		if (i == -1)
			return price;
		int l = price.length();
		return price.substring(0, i) + price.substring(i + 2, l);
	}
	protected int isAlreadyMangel(String ID, Haendler haendlerInstance) {
		Vector m = haendlerInstance.getMangelList();
		if (m.isEmpty()) {
			return -1;
		}
		String BestellID = "";
		int i;
		MangelEintrag mangel = null;
		for (i = 0; i < m.size(); i++) {
			mangel = (MangelEintrag) m.get(i);
			BestellID = mangel.getBestellID();
			if (BestellID.equals(ID)) {
				mangel.incFehlerZahl();
				return i;
			}
		}
		return -1;
	}
	protected void readData(Document doc, Haendler haendlerInstance) {
		// Erstmal testen...
		Document[] docs = haendlerInstance.verify(doc, context);
		if ((docs == null) || (docs.length != 1)) {
			Status = false;
			if (docs.length == 0) {
				ErrorMsg = "Mangel nicht signiert.";
			} else {
				ErrorMsg = "Mangel verletzt Protokoll.";
			}
			return;
		}
		doc = docs[0];
		//Treuhaender-Daten auslesen
		NodeList treuhaenderList = doc.getElementsByTagName("Treuhaender");
		if (treuhaenderList.getLength() == 0) {
			ErrorMsg = "Mangel ... kein Treuhaender gefunden";
			Status = false;
			return;
		}
		Node thNode = treuhaenderList.item(0);
		String TName = "";
		String TAdresse = "";
		String TTelefon = "";
		String TIPAdresse = "";
		String TPort = "";
		if (thNode.hasChildNodes()) {
			thNode = thNode.getFirstChild();
			do {
				if (thNode.getNodeName().equals("TName"))
					TName = thNode.getFirstChild().getNodeValue();
				if (thNode.getNodeName().equals("TAdresse"))
					TAdresse = thNode.getFirstChild().getNodeValue();
				if (thNode.getNodeName().equals("TTelefon"))
					TTelefon = thNode.getFirstChild().getNodeValue();
				if (thNode.getNodeName().equals("TIPAdresse"))
					TIPAdresse = thNode.getFirstChild().getNodeValue();
				if (thNode.getNodeName().equals("TPort"))
					TPort = thNode.getFirstChild().getNodeValue();
				thNode = thNode.getNextSibling();
			} while (thNode != null);
		}
		neuerMangel.setTreuhaender(
			TName,
			TAdresse,
			TTelefon,
			TIPAdresse,
			TPort);
		//	System.out.println("Mangel ... Treuhaender-Pseudonym gefunden: "+ neuerMangel.getTName());

		//Haendler-Daten auslesen und vergleichen
		NodeList haendlerList = doc.getElementsByTagName("Haendler");
		if (haendlerList.getLength() == 0) {
			ErrorMsg = "Mangel ... kein Haendler gefunden";
			Status = false;
			return;
		}
		Node haendlerNode = haendlerList.item(0);
		String HName = "";
		String HIPAdresse = "";
		String HPort = "";
		if (haendlerNode.hasChildNodes()) {
			haendlerNode = haendlerNode.getFirstChild();
			do {
				if (haendlerNode.getNodeName().equals("HName")) {
					HName = haendlerNode.getFirstChild().getNodeValue();
				}
				if (haendlerNode.getNodeName().equals("HIPAdresse")) {
					HIPAdresse = haendlerNode.getFirstChild().getNodeValue();
				}
				if (haendlerNode.getNodeName().equals("HPort")) {
					HPort = haendlerNode.getFirstChild().getNodeValue();
				}
				haendlerNode = haendlerNode.getNextSibling();
			} while (haendlerNode != null);
		}
		neuerMangel.setHaendler(HName, HIPAdresse, HPort);
		//	System.out.println("Mangel ... Haendler-Pseudonym gefunden: "+ neuerMangel.getHName());

		/* Es war einmal
		if (!haendlerInstance.verify(doc, neuerMangel.getTName())) {
		    System.out.println("Mangel: Signatur FALSCH ...");
		    ErrorMsg = "Mangel ... Validierung des Treuhaender-Pseudonym fehlgeschlagen";
		    Status=false;
		    return;
		}
		*/

		//Bestell_ID auslesen
		NodeList bestellIDList = doc.getElementsByTagName("Bestell_ID");
		if (bestellIDList.getLength() == 0) {
			ErrorMsg = "Mangel ... Keine Bestell_ID angegeben";
			Status = false;
			return;
		} else {
			neuerMangel.setBestellID(
				bestellIDList.item(0).getFirstChild().getNodeValue());
			//	   System.out.println("Mangel ... Bestell_ID: " + neuerMangel.getBestellID());
		}

		//Ges_Preis auslesen
		NodeList PreisList = doc.getElementsByTagName("Ges_Preis");
		if (PreisList.getLength() == 0) {
			ErrorMsg = "Mangel ... Keine Ges_Preis angegeben";
			Status = false;
			return;
		} else {
			String gesPreis =
				parsePrice(PreisList.item(0).getFirstChild().getNodeValue());
			neuerMangel.setGesPreis(gesPreis);
			//	   System.out.println("Mangel ... GesPreis: " + neuerMangel.getGesPreis());
		}

	}
	protected int checkBestellung(Haendler haendlerInstance) {

		Vector b = haendlerInstance.getBestellList();
		if (b.isEmpty()) {
			Status = false;
			ErrorMsg = "Mangel ... Es liegen keine Bestellungen vor";
			return -1;
		}
		Bestellung.BestellSatz bestellung = null;
		String BestellID = "";
		int i;
		for (i = 0; i < b.size(); i++) {
			bestellung = (Bestellung.BestellSatz) b.get(i);
			BestellID = bestellung.getBestellID();
			System.out.println("MangelListe: " + BestellID);
			if (BestellID.equals(neuerMangel.getBestellID()))
				break;
		}
		// Ges_Preise vergleichen ...
		if (bestellung.getGesPreis().equals(neuerMangel.getGesPreis())) {
			//		System.out.println("Mangel: Gesamtpreis stimmt");
			mangelBestellung = bestellung;
			return i;
		} else {
			Status = false;
			ErrorMsg =
				"Mangel ... Gesamtpreise von Bestellung und Mangel stimmen nicht ueberein";
			return -1;
		}
	}
	protected void checkMangel(Haendler haendlerInstance) {
		int index;
		Vector m = haendlerInstance.getMangelList();
		if ((index =
			isAlreadyMangel(neuerMangel.getBestellID(), haendlerInstance))
			>= 0) {
			MangelEintrag existingMangel = (MangelEintrag) m.elementAt(index);
			if (existingMangel.getFehlerZahl() < 4) {
				System.out.println(
					"Mangel: "
						+ existingMangel.getBestellID()
						+ "; Fehlerzahl: "
						+ existingMangel.getFehlerZahl());
				existingMangel.incFehlerZahl();
			} else {
				Status = false;
				ErrorMsg = "Mangel ... maximum Fehlerzahl von 4 berschritten";
			}
		} else {
			MangelEintrag neuerMangelEintrag =
				new MangelEintrag(neuerMangel.getBestellID());
			haendlerInstance.addMangelEintrag(neuerMangelEintrag);
		}
	}
	protected void repeatBestellung(Haendler haendlerInstance) {
		Bestellung neueBestellung =
			new Bestellung(mangelBestellung, haendlerInstance, context);
		mangel = neueBestellung.getBestellung();
		if (mangel == null) {
			Status = false;
			ErrorMsg = neueBestellung.getError();
		}
	}

	public static class MangelEintrag {

		private String BestellID;
		private int FehlerZahl;

		public MangelEintrag(String ID) {
			BestellID = ID;
			FehlerZahl = 1;
		}

		public String getBestellID() {
			return BestellID;
		}

		public int getFehlerZahl() {
			return FehlerZahl;
		}

		public void incFehlerZahl() {
			FehlerZahl++;
		}
	}

	public static class MangelSatz {

		private String BestellID;
		private String TName;
		private String TAdresse;
		private String TTelefon;
		private String TIPAdresse;
		private String TPort;
		private String HName;
		private String HIPAdresse;
		private String HPort;
		private String GesPreis;
		private String Begruendung;

		public MangelSatz() {
		}
		protected void setBestellID(String BID) {
			BestellID = BID;
		}
		protected String getBestellID() {
			return BestellID;
		}
		protected void setTreuhaender(
			String TN,
			String TAdr,
			String TTel,
			String TIP,
			String TP) {
			TName = TN;
			TAdresse = TAdr;
			TTelefon = TTel;
			TIPAdresse = TIP;
			TPort = TP;
		}
		protected void setTName(String TN) {
			TName = TN;
		}
		protected void setTAdresse(String TAdr) {
			TAdresse = TAdr;
		}
		protected void setTTelefon(String TTel) {
			TTelefon = TTel;
		}
		protected void setTIPAdresse(String TIP) {
			TIPAdresse = TIP;
		}
		protected void setTPort(String TP) {
			TPort = TP;
		}
		protected String getTName() {
			return TName;
		}
		protected String getTAdresse() {
			return TAdresse;
		}
		protected String getTTelefon() {
			return TTelefon;
		}
		protected String getTIPAdresse() {
			return TIPAdresse;
		}
		protected String getTPort() {
			return TPort;
		}
		protected void setHaendler(String HN, String HIP, String HP) {
			HName = HN;
			HIPAdresse = HIP;
			HPort = HP;
		}
		protected void setHName(String HN) {
			HName = HN;
		}
		protected String getHName() {
			return HName;
		}
		protected void setHIPAdresse(String HIP) {
			HIPAdresse = HIP;
		}
		protected void setHPort(String HP) {
			HPort = HP;
		}
		protected String getHIPAdresse() {
			return HIPAdresse;
		}
		protected String getHPort() {
			return HPort;
		}
		protected void setGesPreis(String GP) {
			GesPreis = GP;
		}
		protected String getGesPreis() {
			return GesPreis;
		}
		protected void setBegruendung(String Begr) {
			Begruendung = Begr;
		}
		protected String getBegruendung() {
			return Begruendung;
		}
	}
}
