/**
 * @version working
 * @author Thomas Kriegelstein
 * $Id: ClientCustomerRMI.java,v 1.5 2004/02/10 16:20:33 tk4 Exp $
 */
package kunde;

import idman.mngt.RemoteSocketFactory;
import idman.rmi.RemoteContext;
import idman.rmi.RemoteManagement;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.rmi.server.RMIClientSocketFactory;
import java.util.Properties;

/**
 * Diese Klasse stellt auf Seite des Kunden eine Verbindung mit
 * einem Server unter Benutzung eines ber RMI ansprechbaren Managers her.
 * @author Thomas Kriegelstein
 * @version $Id: ClientCustomerRMI.java,v 1.5 2004/02/10 16:20:33 tk4 Exp $
 */
public class ClientCustomerRMI {

	private Socket socket;
	private InputStream is;
	private OutputStream os;

	private RemoteContext remote;
	private RemoteManagement manager;
	private RemoteSocketFactory factory;
	private Properties settings;

	public boolean connect() {
		try {
			int port = manager.prepareConnection(remote);
			String host = settings.getProperty("rmi.registry");
			RMIClientSocketFactory csf = factory.createClientSocketFactory();
			socket = csf.createSocket(host, port);

			os = socket.getOutputStream();
			is = socket.getInputStream();
			return true;
		} catch (IOException ioe) {
			System.out.println(
				"ClientCustomerRMI#connect: " + ioe.getMessage());
			try {
				socket.close();
			} catch (Exception e) {
			}
			return false;
		}
	}
	public void disconnect() {
		try {
			is.close();
			os.close();
			socket.close();
		} catch (IOException ioe) {
			System.out.println(
				"ClientCustomerRMI#disconnect: " + ioe.getMessage());
			try {
				socket.close();
			} catch (Exception e) {
			}
		}
	}
	public ClientCustomerRMI(
		RemoteContext remote,
		RemoteManagement manager,
		RemoteSocketFactory factory,
		Properties settings) {
		this.remote = remote;
		this.manager = manager;
		this.factory = factory;
		this.settings = settings;
	}
	// wird vom Kunden aufgerufen --> Bestellung
	public byte[] perform(byte[] order) {
		sendRequest(order);
		byte[] response = getResponse();
		return response;
	}
	private void sendRequest(byte[] request) {
		try {
			byte[] tmp = request;
			byte[] len =
				{
					(byte) (tmp.length >> 24 & 255),
					(byte) (tmp.length >> 16 & 255),
					(byte) (tmp.length >> 8 & 255),
					(byte) (tmp.length & 255)};
			os.write(len);
			os.write(tmp);
			os.flush();
		} catch (IOException ioe) {
			System.out.println(
				"ClientCustomerRMI#sendRequest: " + ioe.getMessage());
		}
	}
	private byte[] getResponse() {
		try {
			int i, len = 0;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			if (len == 0)
				return null;
			byte[] tmp = new byte[len];
			for (int c = 0, j = 0;
				(c < len) && (-1 != (j = is.read(tmp, c, len - c)));
				c += j);

			return tmp;
		} catch (Exception e) {
			System.out.println(
				"ClientCustomerRMI#getResponse: " + e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
}
