/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: Customer.java,v 1.4 2004/02/10 16:20:33 tk4 Exp $
 */
package kunde;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.border.Border;

/**
 * Verwaltung und Anzeige des Kunden-Mens.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: Customer.java,v 1.4 2004/02/10 16:20:33 tk4 Exp $
 */
public class Customer extends JMenu implements ActionListener {

	private final JPanel hostPanel = new JPanel(new BorderLayout());
	private final JPanel portPanel = new JPanel(new BorderLayout());
	private final JPanel namePanel = new JPanel(new BorderLayout());
	private final JPanel infoPanel = new JPanel(new BorderLayout());

	private final Border hostBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Name");
	private final Border portBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Port ?!?");
	private final Border nameBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Passwort");
	private final Border infoBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Info");

	private final JTextField hostField = new JTextField("kunde1");
	private final JTextField portField = new JTextField("3000");
	private final JTextField nameField = new JTextField("kunde1");
	private final JLabel infoField = new JLabel(" ");

	private final JButton okButton = new JButton("Ok");
	private final JButton noButton = new JButton("Cancel");

	private JMenuItem clearItem = new JMenuItem("Katalog lschen");
	private JMenuItem undoItem = new JMenuItem("Bestellung zurcksetzen");
	private JMenuItem orderItem = new JMenuItem("Bestellen");
	private JMenuItem dataItem = new JMenuItem("Kundendaten");

	private final MainFrame mainFrame;
	private final JFrame dataFrame;

	public Customer(MainFrame mainFrame) {
		this.setText("Kunde");
		this.add(clearItem);
		this.addSeparator();
		this.add(undoItem);
		this.add(orderItem);
		//this.addSeparator();
		//this.add(dataItem);
		clearItem.addActionListener(this);
		undoItem.addActionListener(this);
		orderItem.addActionListener(this);
		dataItem.addActionListener(this);
		this.mainFrame = mainFrame;
		this.dataFrame = new JFrame("Kunde");
		initComponents();
	}

	/** Dispatch events. */
	public void actionPerformed(ActionEvent ae) {
		Object source = ae.getSource();
		if (source == clearItem)
			clearItemActionPerformed();
		if (source == undoItem)
			undoItemActionPerformed();
		if (source == orderItem)
			orderItemActionPerformed();
		if (source == dataItem)
			dataItemActionPerformed();
		if (source == okButton)
			okButtonActionPerformed();
		if (source == noButton)
			noButtonActionPerformed();
	}

	private void clearItemActionPerformed() {
		mainFrame.clearActionPerformed();
	}

	private void orderItemActionPerformed() {
		mainFrame.orderActionPerformed();
	}

	private void undoItemActionPerformed() {
		mainFrame.undoActionPerformed();
	}

	private void dataItemActionPerformed() {
		//Point pos = mainFrame.getLocationOnScreen();
		//Insets  i = mainFrame.getInsets();
		hostField.setText(mainFrame.customerData.getPassword());
		portField.setText("" + mainFrame.customerData.getPortNumber());
		nameField.setText(mainFrame.customerData.getPseudonym());
		infoField.setText(" ");
		//dataFrame.setBounds(pos.x+i.left, pos.y+i.top, 360, 290);
		dataFrame.show();
	}

	private void okButtonActionPerformed() {
		int portNumber;
		String host = hostField.getText();
		String port = portField.getText();
		String name = nameField.getText();

		if (host.length() < 1) {
			infoField.setText("Name darf nicht leer bleiben.");
			return;
		}
		if (port.length() < 1) {
			infoField.setText("Port darf nicht leer bleiben.");
			return;
		}
		if (name.length() < 1) {
			infoField.setText("Passwort darf nicht leer bleiben.");
			return;
		}
		try {
			portNumber = Integer.parseInt(port);
		} catch (NumberFormatException nfe) {
			portNumber = -1;
		}
		if (portNumber < 1) {
			infoField.setText("Port mu eine Zahl sein.");
			return;
		}
		mainFrame.customerData = new PrivateData(portNumber, name, host);
		dataFrame.setVisible(false);
	}

	private void noButtonActionPerformed() {
		dataFrame.setVisible(false);
	}

	private void initComponents() {
		dataFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		dataFrame.getContentPane().setLayout(new GridBagLayout());

		GridBagConstraints constraints;
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(20, 20, 5, 20);

		hostPanel.setBorder(hostBorder);
		hostPanel.add(hostField);
		dataFrame.getContentPane().add(hostPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		portPanel.setBorder(portBorder);
		portPanel.add(portField);
		//	dataFrame.getContentPane().add(portPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		namePanel.setBorder(nameBorder);
		namePanel.add(nameField);
		dataFrame.getContentPane().add(namePanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 3;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		infoPanel.setBorder(infoBorder);
		infoPanel.add(infoField);
		dataFrame.getContentPane().add(infoPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 4;
		constraints.weightx = 100;
		constraints.insets = new Insets(5, 20, 20, 5);

		okButton.addActionListener(this);

		dataFrame.getContentPane().add(okButton, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 1;
		constraints.gridy = 4;
		constraints.weightx = 100;
		constraints.insets = new Insets(5, 5, 20, 20);

		noButton.addActionListener(this);

		dataFrame.getContentPane().add(noButton, constraints);

		dataFrame.pack();
	}
}
