/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: MainFrame.java,v 1.13 2004/02/11 07:55:08 tk4 Exp $
 */
package kunde;

import idman.dbi.Database;
import idman.gui.LocalManagementGUI;
import idman.mngt.Context;
import idman.mngt.Management;
import idman.rules.ChainEvaluationListener;
import idman.rules.DefaultCertificateListener;
import idman.rules.SimpleChainEvaluationListener;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Enumeration;
import java.util.Properties;

import javax.swing.*;
import javax.swing.border.Border;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ssonet.net.*;
import org.ssonet.util.Preload;
import org.w3c.dom.*;

/**
 * Kunde, Oberflche und zentrale Verarbeitung.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: MainFrame.java,v 1.13 2004/02/11 07:55:08 tk4 Exp $
 */
public class MainFrame
	extends JFrame
	implements ActionListener, Runnable, CertificateListener {
		
	public static boolean debug = false;

	protected final LogModel logModel = new LogModel();
	protected final JTable status = new JTable(logModel);
	protected final JTextArea description = new JTextArea("");

	protected final TheCatalog catalog = new TheCatalog(this);
	protected final MySelectedItems selection = new MySelectedItems(this);

	private final JScrollPane catalogPane = new JScrollPane(catalog);
	private final JScrollPane selectionPane = new JScrollPane(selection);
	private final JScrollPane statusPane = new JScrollPane(status);
	private final JScrollPane descriptionPane = new JScrollPane(description);

	protected final JButton addButton = new JButton("->");
	protected final JButton removeButton = new JButton("<-");
	protected final JButton orderButton = new JButton("Bestellen");

	private final JPanel catalogPanel = new JPanel(new BorderLayout());
	private final JPanel selectionPanel = new JPanel(new BorderLayout());
	private final JPanel statusPanel = new JPanel(new BorderLayout());
	private final JPanel descriptionPanel = new JPanel(new BorderLayout());
	private final JPanel actionPanel = new JPanel(new GridLayout(0, 1));
	private final JPanel orderPanel = new JPanel(new GridLayout(0, 1));

	protected final JTextField numberField = new JTextField("1");
	protected final JTextField moneyField = new JTextField("");

	private final Border catalogBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Katalog");
	private final Border selectionBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Warenkorb");
	private final Border statusBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Statusmeldungen");
	private final Border descriptionBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Weitere Informationen");
	private final Border orderBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Bestellen");

	private final JLabel countLabel = new JLabel("Anzahl");
	private final JLabel moneyLabel = new JLabel("berwiesenes Geld");
	private final JLabel unitLabel = new JLabel(" DM");
	protected final JLabel totalLabel = new JLabel("Gesamt : 0,00 DM");

	private final MyMenuBar menuBar = new MyMenuBar(this);

	PrivateData customerData = new PrivateData(3000, "Kunde", "kunde1");
	PrivateData traderData = new PrivateData(4013, "Kunde", "localhost");
	PrivateData trusteeData = new PrivateData(4012, "Kunde", "localhost");

	protected Properties settings = null;
	protected Management manager = null;
	protected CertificateListener listener = null;
	protected ChainEvaluationListener chainer = null;
	protected LocalManagementGUI ui = null;

	protected Thread catalogThread = null;
	protected Thread orderThread = null;

	protected static DocumentBuilderFactory dbf = null;
	protected static DocumentBuilder db = null;
	protected static TransformerFactory tf = null;
	protected static Transformer t = null;

	/** Initialisierung der XML-Verarbeitung. */
	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
			tf = TransformerFactory.newInstance();
			t = tf.newTransformer();
		} catch (ParserConfigurationException pce) {
			System.err.println(
				"MainFrame<init>: Could not create "
					+ "Factory"
					+ pce.getMessage());
			pce.printStackTrace();
		} catch (TransformerConfigurationException tce) {
			System.err.println(
				"MainFrame<init>: Could not create "
					+ "Factory"
					+ tce.getMessage());
			tce.printStackTrace();
		} /*catch(TransformerException te) {
							    System.err.println("MainFrame<init>: Could not create "+
									       "Transformer" + te.getMessage());
							    te.printStackTrace();
							} */
	}

	public void init(Properties settings) throws Exception {
		Preload preload = new Preload();
		manager.setStatus("Verbinde zu Datenbank");
		Database.getDBI(settings);
		manager.setStatus("ffne GUI");
		try {
			JFrame frame = new JFrame("Kunde - Lokal");
			this.ui = new LocalManagementGUI(manager, frame, settings);
			frame.getContentPane().add(ui);
			frame.setSize(400, 400);
			frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			frame.show();
		} catch (Exception e) {
			System.err.println("Could not open GUI: " + e.getMessage());
		}
		manager.setStatus("Warte auf Sicherheitssubsystem");
		preload.join();
		manager.setStatus("Kunde bereit.");
	}

	public MainFrame(Properties settings) {
		super("Kunde");
		this.settings = settings;
		this.manager = new idman.mngt.impl.LocalManagement(settings);
		this.chainer = new SimpleChainEvaluationListener(this);
		this.listener = new DefaultCertificateListener(chainer);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		setSize(580, 480);
		initComponents();
	}

	public int checkCertificate(SSONETContext context, X509Certificate cert) {
		String description = "Pseudonym/Zertifikat erhalten";
		boolean trust = CheckCertificateDialog.getAnswer(description, cert);
		if (trust)
			return VALID;
		else
			return INVALID;
	}

	public Context createContext(String rolename, String action) {
		Context result = manager.createContext(rolename);
		result.setAction(action);
		result.addCertificateListener(this);
		result.addCertificateListener(listener);
		result.setOwnActionConfiguration(new ActionConfiguration(3, 3, 0, 0));
		System.out.println("Context created.");
		return result;
	}

	private void initComponents() {
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc;

		catalogPanel.setBorder(catalogBorder);
		catalogPanel.add(catalogPane);
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 75;
		gbc.weighty = 50;
		gbc.insets = new Insets(20, 20, 5, 5);
		getContentPane().add(catalogPanel, gbc);

		descriptionPanel.setBorder(descriptionBorder);
		descriptionPanel.add(descriptionPane);
		description.setRows(5);
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 75;
		gbc.insets = new Insets(5, 20, 5, 5);
		getContentPane().add(descriptionPanel, gbc);

		statusPanel.setBorder(statusBorder);
		statusPanel.add(statusPane);
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 3;
		gbc.weightx = 100;
		gbc.weighty = 70;
		gbc.insets = new Insets(5, 20, 20, 20);
		getContentPane().add(statusPanel, gbc);

		selectionPanel.setBorder(selectionBorder);
		selectionPanel.add(selectionPane);
		selectionPanel.add(totalLabel, BorderLayout.SOUTH);
		totalLabel.setHorizontalAlignment(JLabel.RIGHT);
		totalLabel.setForeground(Color.red);
		totalLabel.setFont(new Font(null, Font.BOLD, 16));
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 2;
		gbc.gridy = 0;
		gbc.weightx = 25;
		gbc.insets = new Insets(20, 5, 5, 20);
		getContentPane().add(selectionPanel, gbc);

		orderPanel.setBorder(orderBorder);
		orderPanel.add(moneyLabel);
		JPanel tmp = new JPanel(new BorderLayout());
		orderPanel.add(tmp);
		tmp.add(moneyField, BorderLayout.CENTER);
		tmp.add(unitLabel, BorderLayout.EAST);
		orderPanel.add(orderButton);
		orderButton.setEnabled(false);
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 2;
		gbc.gridy = 1;
		gbc.weightx = 25;
		gbc.insets = new Insets(5, 5, 5, 20);
		getContentPane().add(orderPanel, gbc);

		actionPanel.add(addButton);
		actionPanel.add(countLabel);
		actionPanel.add(numberField);
		actionPanel.add(removeButton);
		addButton.setEnabled(false);
		removeButton.setEnabled(false);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.insets = new Insets(40, 5, 40, 5);
		getContentPane().add(actionPanel, gbc);

		addButton.addActionListener(this);
		removeButton.addActionListener(this);
		orderButton.addActionListener(this);

		setJMenuBar(menuBar);
	}
	/** Dispatch events. */
	public void actionPerformed(ActionEvent ae) {
		Object s = ae.getSource();
		if (s == addButton)
			addActionPerformed();
		if (s == removeButton)
			removeActionPerformed();
		if (s == orderButton)
			orderActionPerformed();
	}
	// Overridden so we can exit when window is closed
	protected void processWindowEvent(WindowEvent we) {
		super.processWindowEvent(we);
		if (we.getID() == WindowEvent.WINDOW_CLOSING) {
			try {
				Database.ungetDBI();
			} catch (Exception e) {
			}
			System.exit(0);
		}
	}
	/** Called from TheCatalog and from Button. */
	public void addActionPerformed() {
		String s = numberField.getText();
		int number;
		try {
			number = Integer.parseInt(s);
		} catch (NumberFormatException nfe) {
			number = -1;
		}
		if (number < 1) {
			numberField.setText("1");
			number = 1;
		}
		int index = catalog.getSelectedIndex();
		Node node = catalog.getNode(index);
		String text = catalog.getSelectedItem();
		selection.add(index, node, text, number);
		removeButton.setEnabled(false);
		selection.setLastSelectedIndex(-1);
	}
	/** Called from Button. */
	protected void removeActionPerformed() {
		String s = numberField.getText();
		int number;
		try {
			number = Integer.parseInt(s);
		} catch (NumberFormatException nfe) {
			number = -1;
		}
		if (number < 1) {
			numberField.setText("1");
			number = 1;
		}
		int index = selection.getSelectedIndex();
		selection.sub(index, number);
	}

	/** Called from Customer and from Button. */
	public synchronized void orderActionPerformed() {
		if (orderThread != null)
			return;
		else {
			orderThread = new Thread(this);
			orderThread.start();
		}
	}

	public void run() {
		Thread me = Thread.currentThread();
		if (me == catalogThread) {
			realyLoadCatalog();
			catalogThread = null;
		} else if (me == orderThread) {
			realyOrder();
			orderThread = null;
		}
	}

	/** Called from run. */
	private void realyOrder() {
		try {
			Document document = null;
			log(new Date(), "Erstelle Bestellung", "");
			if (selection.getItemCount() == 0) {
				log(new Date(), "Bestellung waere leer", "Abbruch");
				return;
			}
			orderButton.setEnabled(false);
			getJMenuBar().getMenu(0).getItem(3).setEnabled(false);

			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			DocumentBuilder db = dbf.newDocumentBuilder();
			document = db.newDocument();

			Element bestellung = document.createElement("Bestellung");
			Element kunde = document.createElement("Kunde");
			Element kname = document.createElement("KName");
			Element kipadresse = document.createElement("KIPAdresse");
			Element kport = document.createElement("KPort");

			Element haendler = document.createElement("Haendler");
			Element hname = document.createElement("HName");
			Element hipadresse = document.createElement("HIPAdresse");
			Element hport = document.createElement("HPort");

			Element treuhaender = document.createElement("Treuhaender");
			Element tname = document.createElement("TName");
			Element tadresse = document.createElement("TAdresse");
			Element ttelefon = document.createElement("TTelefon");
			Element tipadresse = document.createElement("TIPAdresse");
			Element tport = document.createElement("TPort");

			document.appendChild(bestellung);

			bestellung.appendChild(kunde);
			bestellung.appendChild(haendler);
			bestellung.appendChild(treuhaender);
			kunde.appendChild(kname);
			kunde.appendChild(kipadresse);
			kunde.appendChild(kport);
			haendler.appendChild(hname);
			haendler.appendChild(hipadresse);
			haendler.appendChild(hport);
			treuhaender.appendChild(tname);
			treuhaender.appendChild(tadresse);
			treuhaender.appendChild(ttelefon);
			treuhaender.appendChild(tipadresse);
			treuhaender.appendChild(tport);

			kname.appendChild(
				document.createTextNode(customerData.getPseudonym()));
			kipadresse.appendChild(document.createTextNode("localhost"));
			kport.appendChild(
				document.createTextNode("" + customerData.getPortNumber()));

			tname.appendChild(
				document.createTextNode(trusteeData.getPseudonym()));
			tadresse.appendChild(document.createTextNode("Adresse"));
			ttelefon.appendChild(document.createTextNode("0815/77777"));
			tipadresse.appendChild(
				document.createTextNode(
					InetAddress
						.getByName(trusteeData.getPassword())
						.getHostAddress()));
			tport.appendChild(
				document.createTextNode("" + trusteeData.getPortNumber()));

			hname.appendChild(
				document.createTextNode(traderData.getPseudonym()));
			hipadresse.appendChild(
				document.createTextNode(
					InetAddress
						.getByName(traderData.getPassword())
						.getHostAddress()));
			hport.appendChild(
				document.createTextNode("" + traderData.getPortNumber()));

			Enumeration e = selection.getOrder().elements();
			Node trader = document.importNode((Node)e.nextElement(), true);
			Node catalogNumber =
				document.importNode((Node) e.nextElement(), true);

			bestellung.appendChild(catalogNumber);

			Element gesamtBestellung =
				document.createElement("Gesamtbestellung");
			bestellung.appendChild(gesamtBestellung);
			while (e.hasMoreElements()) {
				Node entry = (Node) e.nextElement();
				int count = 0;
				try {
					if (debug) t.transform(new DOMSource(entry), new StreamResult(System.out));
					count = ((Integer) e.nextElement()).intValue();
				} catch (Exception ex) {
					log(new Date(), "Cast (Integer) failed.", ex.toString());
				}
				for (int k = 0; k < count; k++) {
					Element eintrag = document.createElement("Eintrag");
					gesamtBestellung.appendChild(eintrag);
					NodeList nl = entry.getChildNodes();
					Element id = document.createElement("ID");
					Element rp = document.createElement("Preis");
					for (int i = 0; i < nl.getLength(); i++) {
						if (nl.item(i).getNodeName().equals("ID"))
							id.appendChild(
								document.createTextNode(
									nl.item(i).getFirstChild().getNodeValue()));
						if (nl.item(i).getNodeName().equals("Preis"))
							rp.appendChild(
								document.createTextNode(
									nl.item(i).getFirstChild().getNodeValue()));
					}
					eintrag.appendChild(id);
					eintrag.appendChild(rp);
				}
			}
			Element gesamtPreis = document.createElement("Ges_Preis");
			Element geld = document.createElement("Geld");
			String sum =
				""
					+ (selection.getSum() / 100)
					+ ","
					+ (selection.getSum() % 100);

			gesamtPreis.appendChild(document.createTextNode("DM " + sum));
			geld.appendChild(
				document.createTextNode("DM " + moneyField.getText()));

			bestellung.appendChild(gesamtPreis);
			bestellung.appendChild(geld);

			log(new Date(), "Bestellung erstellt", "");

			// CONTEXT ERSTELLEN
			String hostname = trusteeData.getPassword();
			int port = trusteeData.getPortNumber();
			String rolename = trusteeData.getPseudonym();
			Context context = createContext(rolename, "Bestellung");
			InetAddress target = null;
			try {
				target = InetAddress.getByName(hostname);
			} catch (UnknownHostException uhe) {
			}
			// TODO: Fehlerbehandlung!
			if (target == null) {
				log(new Date(), "Partner nicht gefunden", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			// Fuellen
			context.setTargetAddress(target);
			context.setTargetPort(port);
			// VERBINDUNG HERSTELLEN
			ClientCustomer cc = new ClientCustomer(context);
			// SSONET HANDSHAKE
			boolean ok = cc.connect();
			// TODO: Fehlerbehandlung!
			if (!ok) {
				log(new Date(), "Verbindungsaufbau fehlgeschlagen", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			manager.registerContext(context);
			log(new Date(), "Verbindung hergestellt", "");
			// DOKUMENT SIGNIEREN
			Document signedDocument = null;
			signedDocument = manager.signDocument(document, context);
			log(new Date(), "Bestellung signiert", "");
			// DOKUMENT SENDEN
			log(new Date(), "Sende Bestellung", "");
			signedDocument = cc.perform(signedDocument);
			// TODO: Fehlerbehandlung!
			if (signedDocument == null) {
				log(new Date(), "Keine Antwort erhalten", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			// SIGNATUR PRUEFEN
			Document[] referenced =
				manager.verifyDocument(signedDocument, context);
			// TODO: Fehlerbehandlung!
			if (referenced == null || referenced.length != 1) {
				log(
					new Date(),
					"Signatur der Lieferung nicht korrekt",
					"Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			// AUFRAEUMEN
			signedDocument = referenced[0];
			cc.disconnect();
			manager.unregisterContext(context);

			// should be documentElement..
			NodeList nl = signedDocument.getElementsByTagName("Mangelkunde");
			if (nl.getLength() != 0) {
				NodeList error = signedDocument.getElementsByTagName("Status");
				log(
					new Date(),
					error.item(0).getFirstChild().getNodeValue(),
					"Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			log(new Date(), "Lieferung eingetroffen", "Fertig");
			getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
			orderButton.setEnabled(true);
			undoActionPerformed();
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
			orderButton.setEnabled(true);
			e.printStackTrace();
			return;
		}
	}
	/** Called from everywhere. */
	public void log(Date when, String what, String info) {
		if (info == null)
			info = "";
		logModel.log(when, what, info);
		System.out.println(
			when.toString()
				+ ": \t"
				+ what
				+ (info.equals("") ? "" : " (" + info + ")"));
	}
	/** Called from Customer. */
	public void undoActionPerformed() {
		selection.removeAll();
		selection.setLastSelectedIndex(-1);
		selection.resetSum();
		selection.resetList();
		removeButton.setEnabled(false);
		totalLabel.setText("Gesamt : 0,00 DM");
		orderButton.setEnabled(false);
	}
	public void clearActionPerformed() {
		undoActionPerformed();
		catalog.removeAll();
		catalog.setArray(0);
		catalog.setInitialized(false);
		catalog.setLastSelectedItem(-1);
		catalog.setCount(0);
		addButton.setEnabled(false);
		numberField.setText("1");
		numberField.setEnabled(false);
		moneyField.setText("0,00");
		moneyField.setEnabled(false);
		description.setText("");
		orderButton.setEnabled(false);
	}
	/** Called from Trader. */
	public void loadCatalog() {
		if (catalogThread != null)
			return;
		else {
			catalogThread = new Thread(this);
			catalogThread.start();
		}
	}
	/** Called from run. */
	private void realyLoadCatalog() {
		if (catalog.isInitialized()) {
			log(new Date(), "Katalog bereits geladen", "Abbruch");
			return;
		}
		log(new Date(), "Fordere Katalog an", "");

		// Anfrage formulieren
		Document request = null;
		try {
			/*	    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				    dbf.setNamespaceAware(true);
				    DocumentBuilder db = dbf.newDocumentBuilder();
			*/
			request = db.newDocument();

			Element anforderung = request.createElement("Kataloganforderung");
			Element kunde = request.createElement("Kunde");
			Element kname = request.createElement("KName");
			Element kipadresse = request.createElement("KIPAdresse");
			Element kport = request.createElement("KPort");

			kname.appendChild(
				request.createTextNode(customerData.getPseudonym()));
			kipadresse.appendChild(
				request.createTextNode(
					InetAddress.getLocalHost().getHostAddress()));
			kport.appendChild(
				request.createTextNode("" + customerData.getPortNumber()));

			kunde.appendChild(kname);
			kunde.appendChild(kipadresse);
			kunde.appendChild(kport);
			anforderung.appendChild(kunde);

			Element haendler = request.createElement("Haendler");
			Element hname = request.createElement("HName");
			Element hipadresse = request.createElement("HIPAdresse");
			Element hport = request.createElement("HPort");

			hname.appendChild(
				request.createTextNode(traderData.getPseudonym()));
			hipadresse.appendChild(
				request.createTextNode(
					InetAddress
						.getByName(traderData.getPassword())
						.getHostAddress()));
			hport.appendChild(
				request.createTextNode("" + traderData.getPortNumber()));
			haendler.appendChild(hname);
			haendler.appendChild(hipadresse);
			haendler.appendChild(hport);
			anforderung.appendChild(haendler);

			request.appendChild(anforderung);
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			System.out.println("MainFrame#realyLoadCatalog: " + e.getMessage());
			return;
		}
		// Antwort holen
		Document response = null;
		try {
			// CONTEXT ERSTELLEN
			String hostname = traderData.getPassword();
			int port = traderData.getPortNumber();
			String rolename = traderData.getPseudonym();
			Context context = createContext(rolename, "Kataloganforderung");
			InetAddress target = null;
			try {
				target = InetAddress.getByName(hostname);
			} catch (UnknownHostException uhe) {
				log(new Date(), "Fehler: " + uhe.getMessage(), "");
			}
			// Fehlerbehandlung!
			if (target == null) {
				log(new Date(), "Partner nicht gefunden", "Abbruch");
				return;
			}
			// Fuellen
			context.setTargetAddress(target);
			context.setTargetPort(port);
			// VERBINDUNG HERSTELLEN
			ClientCustomer cc = new ClientCustomer(context);
			// SSONET HANDSHAKE
			boolean ok = cc.connect();
			// Fehlerbehandlung!
			if (!ok) {
				log(new Date(), "Verbindungsaufbau fehlgeschlagen", "Abbruch");
				return;
			}
			manager.registerContext(context);
			log(new Date(), "Verbindung hergestellt", "");
			// DOKUMENT SIGNIEREN
			Document signedDocument = null;
			signedDocument = manager.signDocument(request, context);
			log(new Date(), "Anforderung signiert", "");
			// DOKUMENT SENDEN
			signedDocument = cc.perform(signedDocument);
			log(new Date(), "Anforderung abgeschickt", "");
			// Fehlerbehandlung!
			if (signedDocument == null) {
				log(new Date(), "Keine Antwort erhalten", "Abbruch");
				return;
			}
			// SIGNATUR PRUEFEN
			Document[] referenced =
				manager.verifyDocument(signedDocument, context);
			// Fehlerbehandlung!
			if (referenced == null || referenced.length != 1) {
				log(
					new Date(),
					"Signatur des Katalogs nicht korrekt",
					"Abbruch");
				return;
			}
			// AUFRAEUMEN
			response = referenced[0];
			cc.disconnect();
			manager.unregisterContext(context);
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			System.out.println("MainFrame#realyLoadCatalog: " + e.getMessage());
			return;
		}
		// Endverarbeitung:
		NodeList traderList = response.getElementsByTagName("Haendler");
		selection.setTrader(traderList.item(0));
		NodeList catalogNumberList =
			response.getElementsByTagName("Katalog_Nr");
		selection.setCatalogNumber(catalogNumberList.item(0));
		NodeList nodeList = response.getElementsByTagName("Book");
		catalog.setArray(nodeList.getLength());
		Node node;
		for (int i = 0; i < nodeList.getLength(); i++) {
			node = nodeList.item(i);
			String des = "";
			if (node.hasChildNodes()) {
				node = node.getFirstChild();
				do {
					if (node.getNodeName().equals("Titel")
						|| node.getNodeName().equals("Autor"))
						des =
							des + node.getFirstChild().getNodeValue() + " <> ";
					if (node.getNodeName().equals("Preis"))
						des += node.getFirstChild().getNodeValue();
					node = node.getNextSibling();
				} while (node != null);
				catalog.add(des, nodeList.item(i));
			}
		}
		moneyField.setEnabled(true);
		log(new Date(), "Katalog verfgbar", "Fertig");
	}
	public void setDescription(String des) {
		description.setText(des);
	}
	public void clearDescription() {
		description.setText("");
	}
	public void setAddEnabled(boolean b) {
		addButton.setEnabled(b);
	}
	public void setNumberEnabled(boolean b) {
		numberField.setEnabled(b);
	}
	public void setNumber(String num) {
		numberField.setText(num);
	}
	public void setTotal(String amount) {
		totalLabel.setText("Gesamt : " + amount + " DM");
	}
	public void setOrderEnabled(boolean b) {
		orderButton.setEnabled(b);
	}
	public void setRemoveEnabled(boolean b) {
		removeButton.setEnabled(b);
	}
}
