/**
 * @version working
 * @author Thomas Kriegelstein
 * $Id: MainFrameRMI.java,v 1.9 2004/02/10 16:20:33 tk4 Exp $
 */
package kunde;

import idman.mngt.RemoteSocketFactory;
import idman.rmi.RemoteContext;
import idman.rmi.RemoteContextListener;
import idman.rmi.RemoteManagement;
import idman.rmi.impl.RemoteSocketFactoryImpl;

import java.awt.event.WindowEvent;
import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.RemoteObject;
import java.rmi.server.UnicastRemoteObject;
import java.security.cert.X509Certificate;
import java.util.*;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.*;

/**
 * Anpassung des Kunden an die Anbindung des Identittsmanagers per RMI.
 * @author Thomas Kriegelstein
 * @version $Id: MainFrameRMI.java,v 1.9 2004/02/10 16:20:33 tk4 Exp $
 */
public class MainFrameRMI extends MainFrame {

	protected RemoteManagement manager = null;
	protected MyRemoteContextListener mrcl = null;
	protected RemoteSocketFactory factory = null;

	private static class MyRemoteContextListener
		extends RemoteObject
		implements RemoteContextListener {

		private JFrame frame = null;

		public MyRemoteContextListener(JFrame frame) throws RemoteException {
			this.frame = frame;
		}
		public int checkCertificate(RemoteContext rc, X509Certificate cert)
			throws RemoteException {
			try {
				String[] arr = {
				};
				ArrayList l = new ArrayList();
				StringReader sr = new StringReader(cert.toString());
				BufferedReader br = new BufferedReader(sr);
				l.add(
					"Der Kommunikationspartner legte folgendes "
						+ "Zertifikat vor:");
				while (true) {
					try {
						String s = br.readLine();
						if (s == null)
							break;
						l.add(s);
					} catch (Exception e) {
						break;
					}
				}
				l.add("Wollen sie dieses Zertifikat akzeptieren");
				l.add("und die Kommunikation fortsetzen (Ok)");
				l.add("oder die Verbindung abbrechen (Cancel)?");
				arr = (String[]) l.toArray(arr);
				int result =
					JOptionPane.showConfirmDialog(
						frame,
						arr,
						"Zertifikat akzeptieren?",
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE);
				return (result == JOptionPane.OK_OPTION ? 1 : -1);
			} catch (Throwable t) {
				t.printStackTrace();
				return 0;
			}
		}
		public int showWarning(
			RemoteContext rc,
			String[] msg,
			String[] opt,
			int def) {
			int result =
				JOptionPane.showOptionDialog(
					frame,
					msg,
					"Warnung aufgetreten!",
					JOptionPane.DEFAULT_OPTION,
					JOptionPane.WARNING_MESSAGE,
					null,
					opt,
					opt[def]);
			return result;
		}
	}

	public void init(Properties settings) throws Exception {
		String host = settings.getProperty("rmi.registry");
		String url = "//" + host + "/" + RemoteManagement.ID;
		manager = (RemoteManagement) Naming.lookup(url);
	}

	public MainFrameRMI(Properties settings) {
		super(settings);
		setTitle("Kunde - RMI");
		try {
			try {
				Registry reg = LocateRegistry.getRegistry();
				reg.list();
			} catch (Exception e) {
				try {
					LocateRegistry.createRegistry(1099);
				} catch (Exception ignore) {
				}
			}
			factory = new RemoteSocketFactoryImpl(settings);
			mrcl = new MyRemoteContextListener(this);
			UnicastRemoteObject.exportObject(
				mrcl,
				0,
				factory.createClientSocketFactory(),
				factory.createServerSocketFactory());
		} catch (RemoteException re) {
			re.printStackTrace();
		}
	}
	// Overridden so we can exit when window is closed
	protected void processWindowEvent(WindowEvent we) {
		if (we.getID() == WindowEvent.WINDOW_CLOSING) {
			System.exit(0);
		}
		super.processWindowEvent(we);
	}
	public void run() {
		Thread me = Thread.currentThread();
		if (me == catalogThread) {
			realyLoadCatalogRMI();
			catalogThread = null;
		} else if (me == orderThread) {
			realyOrderRMI();
			orderThread = null;
		}
	}
	/** Called from run. */
	private void realyOrderRMI() {
		try {
			Document document = null;
			log(new Date(), "Erstelle Bestellung", "");
			if (selection.getItemCount() == 0) {
				log(new Date(), "Bestellung waere leer", "Abbruch");
				return;
			}
			orderButton.setEnabled(false);
			getJMenuBar().getMenu(0).getItem(3).setEnabled(false);

			document = db.newDocument();

			Element bestellung = document.createElement("Bestellung");
			Element kunde = document.createElement("Kunde");
			Element kname = document.createElement("KName");
			Element kipadresse = document.createElement("KIPAdresse");
			Element kport = document.createElement("KPort");

			Element haendler = document.createElement("Haendler");
			Element hname = document.createElement("HName");
			Element hipadresse = document.createElement("HIPAdresse");
			Element hport = document.createElement("HPort");

			Element treuhaender = document.createElement("Treuhaender");
			Element tname = document.createElement("TName");
			Element tadresse = document.createElement("TAdresse");
			Element ttelefon = document.createElement("TTelefon");
			Element tipadresse = document.createElement("TIPAdresse");
			Element tport = document.createElement("TPort");

			document.appendChild(bestellung);

			bestellung.appendChild(kunde);
			bestellung.appendChild(haendler);
			bestellung.appendChild(treuhaender);
			kunde.appendChild(kname);
			kunde.appendChild(kipadresse);
			kunde.appendChild(kport);
			haendler.appendChild(hname);
			haendler.appendChild(hipadresse);
			haendler.appendChild(hport);
			treuhaender.appendChild(tname);
			treuhaender.appendChild(tadresse);
			treuhaender.appendChild(ttelefon);
			treuhaender.appendChild(tipadresse);
			treuhaender.appendChild(tport);

			kname.appendChild(
				document.createTextNode(customerData.getPseudonym()));
			kipadresse.appendChild(document.createTextNode("localhost"));
			kport.appendChild(
				document.createTextNode("" + customerData.getPortNumber()));

			tname.appendChild(
				document.createTextNode(trusteeData.getPseudonym()));
			tadresse.appendChild(document.createTextNode("Adresse"));
			ttelefon.appendChild(document.createTextNode("0815/77777"));
			tipadresse.appendChild(
				document.createTextNode(
					InetAddress
						.getByName(trusteeData.getPassword())
						.getHostAddress()));
			tport.appendChild(
				document.createTextNode("" + trusteeData.getPortNumber()));

			hname.appendChild(
				document.createTextNode(traderData.getPseudonym()));
			hipadresse.appendChild(
				document.createTextNode(
					InetAddress
						.getByName(traderData.getPassword())
						.getHostAddress()));
			hport.appendChild(
				document.createTextNode("" + traderData.getPortNumber()));

			Enumeration e = selection.getOrder().elements();
			Node catalogNumber =
				document.importNode((Node) e.nextElement(), true);

			bestellung.appendChild(catalogNumber);

			Element gesamtBestellung =
				document.createElement("Gesamtbestellung");
			bestellung.appendChild(gesamtBestellung);
			while (e.hasMoreElements()) {
				Node entry = (Node) e.nextElement();
				int count = ((Integer) e.nextElement()).intValue();
				for (int k = 0; k < count; k++) {
					Element eintrag = document.createElement("Eintrag");
					gesamtBestellung.appendChild(eintrag);
					NodeList nl = entry.getChildNodes();
					Element id = document.createElement("ID");
					Element rp = document.createElement("Preis");
					for (int i = 0; i < nl.getLength(); i++) {
						if (nl.item(i).getNodeName().equals("ID"))
							id.appendChild(
								document.createTextNode(
									nl.item(i).getFirstChild().getNodeValue()));
						if (nl.item(i).getNodeName().equals("Preis"))
							rp.appendChild(
								document.createTextNode(
									nl.item(i).getFirstChild().getNodeValue()));
					}
					eintrag.appendChild(id);
					eintrag.appendChild(rp);
				}
			}
			Element gesamtPreis = document.createElement("Ges_Preis");
			Element geld = document.createElement("Geld");
			String sum =
				""
					+ (selection.getSum() / 100)
					+ ","
					+ (selection.getSum() % 100);

			gesamtPreis.appendChild(document.createTextNode("DM " + sum));
			geld.appendChild(
				document.createTextNode("DM " + moneyField.getText()));

			bestellung.appendChild(gesamtPreis);
			bestellung.appendChild(geld);

			log(new Date(), "Bestellung erstellt", "");

			// CONTEXT ERSTELLEN
			String hostname = trusteeData.getPassword();
			int port = trusteeData.getPortNumber();
			String rolename = trusteeData.getPseudonym();
			InetAddress target = null;
			try {
				target = InetAddress.getByName(hostname);
			} catch (UnknownHostException uhe) {
				log(new Date(), "Fehler: " + uhe.getMessage(), "");
			}
			// Fehlerbehandlung!
			if (target == null) {
				log(new Date(), "Partner nicht gefunden", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			RemoteContext remote =
				manager.createRemoteContext(
					target,
					port,
					rolename,
					"Bestellung");
			// VERBINDUNG HERSTELLEN
			ClientCustomerRMI cc =
				new ClientCustomerRMI(remote, manager, factory, settings);
			manager.addRemoteContextListener(mrcl, remote);
			// SSONET HANDSHAKE
			boolean ok = cc.connect();
			// Fehlerbehandlung!
			if (!ok) {
				log(new Date(), "Verbindungsaufbau fehlgeschlagen", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			log(new Date(), "Verbindung hergestellt", "");
			// DOKUMENT SIGNIEREN
			byte[] signedB = null;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			Transformer t = tf.newTransformer();
			t.transform(new DOMSource(bestellung), new StreamResult(baos));
			byte[] requestB = baos.toByteArray();
			baos.close();
			signedB = manager.signDocument(requestB, remote);
			log(new Date(), "Anforderung signiert", "");
			// DOKUMENT SENDEN
			signedB = cc.perform(signedB);
			log(new Date(), "Anforderung abgeschickt", "");
			// Fehlerbehandlung!
			if (signedB == null) {
				log(new Date(), "Keine Antwort erhalten", "Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			// SIGNATUR PRUEFEN
			byte[][] referencedB = manager.verifyDocument(signedB, remote);
			// Fehlerbehandlung!
			if (referencedB == null || referencedB.length != 1) {
				log(
					new Date(),
					"Signatur der Katalogs nicht korrekt",
					"Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			// AUFRAEUMEN
			Document signedDocument =
				db.parse(new ByteArrayInputStream(referencedB[0]));
			cc.disconnect();
			manager.removeRemoteContextListener(mrcl, remote);
			manager.destroyRemoteContext(remote);
			// should be documentElement..
			NodeList nl = signedDocument.getElementsByTagName("Mangelkunde");
			if (nl.getLength() != 0) {
				NodeList error = signedDocument.getElementsByTagName("Status");
				log(
					new Date(),
					error.item(0).getFirstChild().getNodeValue(),
					"Abbruch");
				getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
				orderButton.setEnabled(true);
				return;
			}
			log(new Date(), "Lieferung eingetroffen", "Fertig");
			getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
			orderButton.setEnabled(true);
			undoActionPerformed();
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			getJMenuBar().getMenu(0).getItem(3).setEnabled(true);
			orderButton.setEnabled(true);
			e.printStackTrace();
			return;
		}
	}
	/** Called from run. */
	private void realyLoadCatalogRMI() {
		if (catalog.isInitialized()) {
			log(new Date(), "Katalog bereits geladen", "Abbruch");
			return;
		}
		log(new Date(), "Fordere Katalog an", "");

		// Anfrage formulieren
		Document request = null;
		try {
			request = db.newDocument();

			Element anforderung = request.createElement("Kataloganforderung");
			Element kunde = request.createElement("Kunde");
			Element kname = request.createElement("KName");
			Element kipadresse = request.createElement("KIPAdresse");
			Element kport = request.createElement("KPort");

			kname.appendChild(
				request.createTextNode(customerData.getPseudonym()));
			kipadresse.appendChild(
				request.createTextNode(
					InetAddress.getLocalHost().getHostAddress()));
			kport.appendChild(
				request.createTextNode("" + customerData.getPortNumber()));

			kunde.appendChild(kname);
			kunde.appendChild(kipadresse);
			kunde.appendChild(kport);
			anforderung.appendChild(kunde);

			Element haendler = request.createElement("Haendler");
			Element hname = request.createElement("HName");
			Element hipadresse = request.createElement("HIPAdresse");
			Element hport = request.createElement("HPort");

			hname.appendChild(
				request.createTextNode(traderData.getPseudonym()));
			hipadresse.appendChild(
				request.createTextNode(
					InetAddress
						.getByName(traderData.getPassword())
						.getHostAddress()));
			hport.appendChild(
				request.createTextNode("" + traderData.getPortNumber()));
			haendler.appendChild(hname);
			haendler.appendChild(hipadresse);
			haendler.appendChild(hport);
			anforderung.appendChild(haendler);

			request.appendChild(anforderung);
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			System.out.println(
				"MainFrameRMI#realyLoadCatalogRMI: " + e.getMessage());
			return;
		}
		// Antwort holen
		Document response = null;
		try {
			// CONTEXT ERSTELLEN
			String hostname = traderData.getPassword();
			int port = traderData.getPortNumber();
			String rolename = traderData.getPseudonym();
			InetAddress target = null;
			try {
				target = InetAddress.getByName(hostname);
			} catch (UnknownHostException uhe) {
				log(new Date(), "Fehler: " + uhe.getMessage(), "");
			}
			// TODO: Fehlerbehandlung!
			if (target == null) {
				log(new Date(), "Partner nicht gefunden", "Abbruch");
				return;
			}
			RemoteContext remote =
				manager.createRemoteContext(
					target,
					port,
					rolename,
					"Kataloganforderung");
			// VERBINDUNG HERSTELLEN
			ClientCustomerRMI cc =
				new ClientCustomerRMI(remote, manager, factory, settings);
			manager.addRemoteContextListener(mrcl, remote);
			// SSONET HANDSHAKE
			boolean ok = cc.connect();
			// TODO: Fehlerbehandlung!
			if (!ok) {
				log(new Date(), "Verbindungsaufbau fehlgeschlagen", "Abbruch");
				return;
			}
			log(new Date(), "Verbindung hergestellt", "");
			// DOKUMENT SIGNIEREN
			byte[] signedB = null;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			Transformer t = tf.newTransformer();
			t.transform(new DOMSource(request), new StreamResult(baos));
			byte[] requestB = baos.toByteArray();
			baos.close();
			signedB = manager.signDocument(requestB, remote);
			log(new Date(), "Anforderung signiert", "");
			// DOKUMENT SENDEN
			signedB = cc.perform(signedB);
			log(new Date(), "Anforderung abgeschickt", "");
			// TODO: Fehlerbehandlung!
			if (signedB == null) {
				log(new Date(), "Keine Antwort erhalten", "Abbruch");
				return;
			}
			// SIGNATUR PRUEFEN
			byte[][] referencedB = manager.verifyDocument(signedB, remote);
			// Fehlerbehandlung!
			if (referencedB == null || referencedB.length != 1) {
				log(
					new Date(),
					"Signatur des Katalogs nicht korrekt",
					"Abbruch");
				return;
			}
			// AUFRAEUMEN
			response = db.parse(new ByteArrayInputStream(referencedB[0]));
			cc.disconnect();
			manager.removeRemoteContextListener(mrcl, remote);
			manager.destroyRemoteContext(remote);
		} catch (Exception e) {
			log(new Date(), "Fehler: " + e.getMessage(), "Abbruch");
			System.out.println(
				"MainFrameRMI#realyLoadCatalogRMI: " + e.getMessage());
			return;
		}
		// Endverarbeitung:
		NodeList traderList = response.getElementsByTagName("Haendler");
		selection.setTrader(traderList.item(0));
		NodeList catalogNumberList =
			response.getElementsByTagName("Katalog_Nr");
		selection.setCatalogNumber(catalogNumberList.item(0));
		NodeList nodeList = response.getElementsByTagName("Book");
		catalog.setArray(nodeList.getLength());
		Node node;
		for (int i = 0; i < nodeList.getLength(); i++) {
			node = nodeList.item(i);
			String des = "";
			if (node.hasChildNodes()) {
				node = node.getFirstChild();
				do {
					if (node.getNodeName().equals("Titel")
						|| node.getNodeName().equals("Autor"))
						des =
							des + node.getFirstChild().getNodeValue() + " <> ";
					if (node.getNodeName().equals("Preis"))
						des += node.getFirstChild().getNodeValue();
					node = node.getNextSibling();
				} while (node != null);
				catalog.add(des, nodeList.item(i));
			}
		}
		moneyField.setEnabled(true);
		log(new Date(), "Katalog verfgbar", "Fertig");
	}
}
