/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: Trustee.java,v 1.4 2004/02/10 16:20:33 tk4 Exp $
 */
package kunde;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Klasse zur Anzeige und Verarbeitung des Treuhndermens.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: Trustee.java,v 1.4 2004/02/10 16:20:33 tk4 Exp $
 */
public class Trustee extends JMenu implements ActionListener {

	private final JPanel hostPanel = new JPanel(new BorderLayout());
	private final JPanel portPanel = new JPanel(new BorderLayout());
	private final JPanel namePanel = new JPanel(new BorderLayout());
	private final JPanel infoPanel = new JPanel(new BorderLayout());

	private final Border hostBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Adresse");
	private final Border portBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Port");
	private final Border nameBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Rolle");
	private final Border infoBorder =
		BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			"Info");

	private final JTextField hostField = new JTextField("127.0.0.1");
	private final JTextField portField = new JTextField("4013");
	private final JTextField nameField = new JTextField("Kunde");
	private final JLabel infoField = new JLabel(" ");

	private final JFrame dataFrame = new JFrame("Treuhnder");

	private final JButton okButton = new JButton("Ok");
	private final JButton noButton = new JButton("Cancel");

	private final JMenuItem dataItem = new JMenuItem("Treuhnderdaten");

	private MainFrame mainFrame = null;

	public Trustee(MainFrame mainFrame) {
		this.setText("Treuhnder");
		this.add(dataItem);
		dataItem.addActionListener(this);
		this.mainFrame = mainFrame;
		initComponents();
	}

	public void actionPerformed(ActionEvent ae) {
		Object source = ae.getSource();
		if (source == dataItem)
			dataItemActionPerformed();
		if (source == okButton)
			okButtonActionPerformed();
		if (source == noButton)
			noButtonActionPerformed();
	}

	private void dataItemActionPerformed() {
		Point pos = mainFrame.getLocationOnScreen();
		Insets i = mainFrame.getInsets();
		hostField.setText(mainFrame.trusteeData.getPassword());
		portField.setText("" + mainFrame.trusteeData.getPortNumber());
		nameField.setText(mainFrame.trusteeData.getPseudonym());
		infoField.setText(" ");
		dataFrame.setBounds(pos.x + i.left, pos.y + i.top, 360, 330);
		dataFrame.setVisible(true);
		dataFrame.show();
	}

	private void okButtonActionPerformed() {
		int portNumber;
		String host = hostField.getText();
		String port = portField.getText();
		String name = nameField.getText();

		if (host.length() < 1) {
			infoField.setText("Adresse darf nicht leer bleiben.");
			return;
		}
		if (port.length() < 1) {
			infoField.setText("Port darf nicht leer bleiben.");
			return;
		}
		if (name.length() < 1) {
			infoField.setText("Rolle darf nicht leer bleiben.");
			return;
		}
		try {
			portNumber = Integer.parseInt(port);
		} catch (NumberFormatException nfe) {
			portNumber = -1;
		}
		if (portNumber < 1) {
			infoField.setText("Port mu eine positive, ganze Zahl sein.");
			return;
		}
		mainFrame.trusteeData = new PrivateData(portNumber, name, host);
		dataFrame.setVisible(false);
	}

	private void noButtonActionPerformed() {
		dataFrame.setVisible(false);
	}

	private void initComponents() {
		dataFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		//dataFrame.setBackground(new Color(255,255,200));
		dataFrame.getContentPane().setLayout(new GridBagLayout());

		GridBagConstraints constraints;
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(20, 20, 5, 20);

		hostPanel.setBorder(hostBorder);
		hostPanel.add(hostField);
		dataFrame.getContentPane().add(hostPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		portPanel.setBorder(portBorder);
		portPanel.add(portField);
		dataFrame.getContentPane().add(portPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		namePanel.setBorder(nameBorder);
		namePanel.add(nameField);
		dataFrame.getContentPane().add(namePanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 3;
		constraints.gridwidth = 2;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(5, 20, 5, 20);

		infoPanel.setBorder(infoBorder);
		infoPanel.add(infoField);
		dataFrame.getContentPane().add(infoPanel, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 4;
		constraints.weightx = 100;
		constraints.insets = new Insets(5, 20, 20, 5);

		okButton.addActionListener(this);

		dataFrame.getContentPane().add(okButton, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 1;
		constraints.gridy = 4;
		constraints.weightx = 100;
		constraints.insets = new Insets(5, 5, 20, 20);

		noButton.addActionListener(this);

		dataFrame.getContentPane().add(noButton, constraints);

		dataFrame.pack();
	}
}
