/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: ClientTrustee.java,v 1.9 2004/02/12 08:12:41 tk4 Exp $
 */
package treuhaender;

import idman.mngt.Context;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ssonet.net.ActionConfiguration;
import org.ssonet.net.ConnectionListener;
import org.ssonet.net.SSONETContext;
import org.ssonet.net.impl.SSONETServerClientSocket;
import org.w3c.dom.Document;

/**
 * Kapselt die Kommunikation via Sockets mit dem Client.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: ClientTrustee.java,v 1.9 2004/02/12 08:12:41 tk4 Exp $
 */
public class ClientTrustee extends Thread {
	static class Waiter implements ConnectionListener {

		/* (non-Javadoc)
		 * @see org.ssonet.net.ConnectionListener#connectionStateChanged(org.ssonet.net.SSONETContext, int)
		 */
		public synchronized void connectionStateChanged(
			SSONETContext context,
			int currentState) {
			if (currentState == ConnectionListener.RUNNING_STATE) {
				notifyAll();
			}
		}
		public synchronized void waitFor(SSONETContext context) {
			if (context.getConnectionState() != RUNNING_STATE) {
				try {
					wait();
				} catch (Exception e) {
				}
			}
		}
		/* (non-Javadoc)
		 * @see org.ssonet.net.ConnectionListener#negotiationRequestReceived(org.ssonet.net.SSONETContext, int)
		 */
		public void negotiationRequestReceived(SSONETContext context, int negotiationType) {
		}
		/* (non-Javadoc)
		 * @see org.ssonet.net.ConnectionListener#decideSecurityGoal(int, org.ssonet.net.ActionConfiguration, org.ssonet.net.SSONETContext)
		 */
		public boolean decideSecurityGoal(int securityGoal, ActionConfiguration ownConfiguration, SSONETContext context) {
			return false;
		}
	}

	private SSONETServerClientSocket s = null;
	private Treuhaender treuhaender = null;
	private Context context = null;

	private static DocumentBuilderFactory dbf = null;
	private static DocumentBuilder db = null;
	private static TransformerFactory tf = null;
	private static Transformer t = null;

	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
			tf = TransformerFactory.newInstance();
			t = tf.newTransformer();
		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		} catch (TransformerConfigurationException tce) {
			tce.printStackTrace();
		} /*catch(TransformerException te) {
				           te.printStackTrace();
				       } */
	}
	/**
	 * Konstruktor.
	 * @param s Zu verarbeitende Verbindung.
	 * @param treuhaender Zentrale Verarbeitungsinstanz.
	 */
	public ClientTrustee(SSONETServerClientSocket s, Treuhaender treuhaender) {
		this.s = s;
		this.treuhaender = treuhaender;
	}

	private Document performRequest(InputStream is) {
		try {
			int i = 0, len = 0;
			System.out.println(i);
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			System.out.println(i);
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			System.out.println(i);
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			System.out.println(i);
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			System.out.println(i);
			if (len == 0)
				return null;
			System.out.println(len);

			byte[] tmp = new byte[len];
			for (int c = 0, j = 0;
				(c < tmp.length)
					&& (-1 != (j = is.read(tmp, c, tmp.length - c)));
				c += j);
			System.out.write(tmp);
			ByteArrayInputStream bais = new ByteArrayInputStream(tmp);

			Document request = db.parse(bais);
			Document response = null;
			response = treuhaender.handleRequest(request, context);
			return response;
		} catch (Exception e) {
			System.out.println(
				"ClientTrustee#performRequest: " + e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
	private void connect() {
		try {
			InputStream is = s.getInputStream();
			OutputStream os = s.getOutputStream();
			this.context = (Context) s.getContext();
			context.setTargetAddress(s.getInetAddress());
			context.setTargetPort(s.getPort());
			System.out.println("ClientTrustee#connect: " + s.toString());
			Document response = performRequest(is);
			if (response != null) {
				//   System.out.println("Einlesen");
				sendResponse(response, os);
			} else {
				os.write(new byte[4]);
			} //else
			os.flush();
			os.close();
			is.close();
			s.close();
		} catch (IOException ioe) {
			System.out.println("ClientTrustee#connect: " + ioe.getMessage());
		}
	}

	// zurckschreiben der Antwort bei erfolgreichem Test
	private void sendResponse(Document response, OutputStream os) {
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			Transformer t = tf.newTransformer();
			t.transform(new DOMSource(response), new StreamResult(baos));

			byte[] tmp = baos.toByteArray();
			baos.close();

			byte[] len =
				{
					(byte) (tmp.length >> 24 & 255),
					(byte) (tmp.length >> 16 & 255),
					(byte) (tmp.length >> 8 & 255),
					(byte) (tmp.length & 255)};
			os.write(len);
			os.write(tmp);
			os.flush();
		} catch (IOException ioe) {
			System.out.println(
				"ClientTrustee#makeResponse: " + ioe.getMessage());
		} catch (TransformerException te) {
			System.out.println(
				"ClientTrustee#makeResponse: " + te.getMessage());
		}
	}
	/**
	 * Arbeitsmethode.
	 */
	public void run() {
		try {
			connect();
		} catch (Exception ie) {
			System.out.println(
				ie.toString() + "   Fehler: Verbindung abgebrochen");
		}
	}
}
