/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: MainClass.java,v 1.2 2004/02/10 16:20:33 tk4 Exp $
 */
package treuhaender;

import idman.dbi.Database;
import idman.util.Configuration;

import java.util.Properties;

import org.ssonet.util.Preload;

/**
 * Startklasse, bernimmt Kommandozeilenauswertung und Start des
 * Treuhnders.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: MainClass.java,v 1.2 2004/02/10 16:20:33 tk4 Exp $
 */
public class MainClass {

	/**
	 * Startmethode.
	 * Parameter:
	 * <ul>
	 * <li>-name &lt;name&gt; Name (Charlie) fr Personenpseudonym.
	 * <li>-role &lt;name&gt; Rollenname (Treuhnder) fr Verbindungen
	 *           zu Kunden.
	 * <li>-port &lt;port&gt; Port (4012) fr Verbindungen zu Kunden.
	 * <li>-help Anzeige eines Hilfetextes.
	 * </ul>
	 * @param args Stringarray mit den Parametern.
	 * throws Exception Im Fehlerfall.
	 */
	public static void main(String[] args) throws Exception {
		args = parseArgs(args);
		Preload preload = new Preload();
		Properties settings = Configuration.getSettings();
		settings.setProperty("mngt.certificate.subjectCN", args[2]);
		Database.getDBI(settings);
		Treuhaender treuhaender = new Treuhaender(settings, args[0], args[1]);
		preload.join();
		treuhaender.start();
	}

	private static String[] parseArgs(String[] args) {
		String[] result = { "Treuhnder", "4012", "Charlie" };
		for (int i = 0; i < args.length; i++) {
			String c = args[i];
			if ("-name".equals(c)) {
				i++;
				if (i == args.length)
					usage();
				else
					result[2] = args[i];
			} else if ("-role".equals(c)) {
				i++;
				if (i == args.length)
					usage();
				else
					result[0] = args[i];
			} else if ("-port".equals(c)) {
				i++;
				if (i == args.length)
					usage();
				else
					result[1] = args[i];
				try {
					int j = Integer.parseInt(result[1]);
					if (j <= 1024)
						System.out.println("Port must be above 1024.");
				} catch (NumberFormatException nfe) {
					System.out.println(
						"Port must be a positive number " + "above 1024.");
				}
			} else {
				usage();
			}
		}
		return result;
	}
	private static void usage() {
		System.out.println(
			"Usage:   [-name <name>] -role [name] [-port <port>]");
		System.out.println(
			"Default: -name Charlie -role Treuhnder -port 4012");
		System.exit(0);
	}
}
