/**
 * @version working
 * @author Sven Doering (lehrprojekt)
 * @author Thomas Kriegelstein
 * $Id: Treuhaender.java,v 1.12 2004/02/10 16:20:33 tk4 Exp $
 */
package treuhaender;

import idman.gui.LocalManagementGUI;
import idman.mngt.Context;
import idman.mngt.impl.LocalManagement;
import idman.rules.ChainEvaluationListener;
import idman.rules.Condition;
import idman.rules.DefaultCertificateListener;
import idman.rules.PseudonymProvider;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.util.Date;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.WindowConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.ssonet.net.ActionConfiguration;
import org.ssonet.net.CertificateListener;
import org.ssonet.net.SSONETContext;
import org.ssonet.net.SSONETContextFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Diese Klasse repraesentiert den Treuhaender in unserem Protokoll
 * fuer rechtsverbindlichen Einkauf im Internet.<br>
 * Grober Ablaufplan im Treuhaender:
 * <ol>
 * <li> Es wurde eine Verbindung zur Netzschnittstelle aufgebaut (vom Kunden)
 * <li> Die empfangenen Daten werden zu einem Document geparst.
 * <li> Es wird die Methode handleRequest(Document) aufgerufen.
 * <li> Auf jeden Fall muss die Signatur ueberprueft werden.
 * <li> nach dem Abhandeln des Requests liefert die Methode handleRequest
 *      ein Document zurueck, welches ueber die Netzschnittstelle zum
 *      urspruenglichen Sender uebertragen wird.
 * </ol>
 *
 * @author Sven Doering (lehrprojekt)
 * @author Thomas Kriegelstein
 * @version $Id: Treuhaender.java,v 1.12 2004/02/10 16:20:33 tk4 Exp $
 */
public class Treuhaender
	implements ChainEvaluationListener, SSONETContextFactory {

	private Properties settings = null;
	private LocalManagement manager = null;
	private ServerTrustee server = null;
	private CertificateListener listener = null;
	private LocalManagementGUI ui = null;

	private String TName_ = "Charlie";
	private String TAdresse_ = "Dresden Stadtwache 9";
	private String TTelefon_ = "0815/4711";
	private String TIPAdresse_ = "127.0.0.1";
	private String TPort_ = "4012";
	private int counter = 1;
	private Hashtable hashtable = new Hashtable();
	private static DocumentBuilderFactory dbf = null;
	private static DocumentBuilder db = null;
	private static MessageDigest digest = null;

	/** Initialisiert die XML-Verarbeitung und den Digest-Algorithmus. */
	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
		} catch (Exception e) {
			System.err.println(
				"Treuhaender<init>: Could not create Factory" + e.getMessage());
			e.printStackTrace();
		}
		try {
			digest = MessageDigest.getInstance("SHA1");
		} catch (Exception e) {
			System.err.println(
				"Treuhaender<init>: Could not create digest: "
					+ e.getMessage());
			e.printStackTrace();
		}
	}
	public boolean doWarning(
		String chainname,
		Condition cond,
		String id,
		byte[] content,
		Context context,
		String duno) {
		System.out.println("Warning in ChainEvaluator.");
		return true;
	}
	/**
	 * Erzeugt einen Context zur Verwendung bei der Kommunikation mit
	 * einem Kunden.
	 * @return Context als SSONETContext mit den Verbindungsdaten.
	 */
	public SSONETContext createContext() {
		Context result = manager.createContext(TName_);
		// nur im Server... Da der den Partner am Anfang eh nicht kennt.
		PseudonymProvider.getPersonalPseudonym(null, null, result);
		result.setOwnActionConfiguration(new ActionConfiguration(3, 3, 0, 0));
		result.addCertificateListener(listener);
		System.out.println("Context created.");
		return result;
	}
	/**
	 * Erzeugt einen Context zur Verwendung bei der Kommunikation mit
	 * einem Hndler.
	 * @param rolename Name der zu verwendenden Rolle.
	 * @param action Auszufhrende Aktion.
	 * @return Context mit den Verbindungsdaten.
	 */
	public Context createContext(String rolename, String action) {
		Context result = manager.createContext(rolename);
		PseudonymProvider.getPersonalPseudonym(null, null, result);
		result.addCertificateListener(listener);
		result.setAction(action);
		// TODO: Name der Aktion auswerten
		result.setOwnActionConfiguration(new ActionConfiguration(3, 3, 0, 0));
		System.out.println("Context created.");
		return result;
	}
	/**
	 * Konstruktor.
	 * @param settings Die Einstellungen.
	 * @param name Name der Rolle fr die Kommunikation mit Kunden.
	 * @param port Port fr die Kommunikation mit Kunden.
	 */
	public Treuhaender(Properties settings, String name, String port) {
		this.settings = settings;
		this.TName_ = name;
		this.TPort_ = port;
		try {
			InetAddress ia = InetAddress.getLocalHost();
			TIPAdresse_ = ia.getHostAddress();
		} catch (UnknownHostException uhe) {
			uhe.printStackTrace();
		}
		System.out.println(
			"Treuhaender: " + TName_ + "/" + TIPAdresse_ + ":" + TPort_);
		int p = 0;
		try {
			p = Integer.parseInt(port);
		} catch (NumberFormatException nfe) {
			// should not happen!
			nfe.printStackTrace();
		}
		this.manager = new LocalManagement(settings);
		this.manager.setStatus("Initialisiere");
		this.server = new ServerTrustee(this, p, this);
		this.listener = new DefaultCertificateListener(this);
		try {
			JFrame frame = new JFrame(TName_);
			this.ui = new LocalManagementGUI(manager, frame, settings);
			frame.getContentPane().add(ui);
			frame.setSize(400, 400);
			frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			frame.show();
		} catch (Error e) {
			System.err.println("Could not open GUI: " + e.getMessage());
		}
	}
	/**
	 * Bereitet die Kommunikation zum Kunden vor.
	 */
	public void start() {
		System.out.println("Treuhaender: gestartet auf: " + TPort_);
		server.start();
		manager.setStatus("Treuhnder bereit.");
	}
	/**
	 * Delegiert das berprfen eines Dokuments an den Manager weiter.
	 * @param d Zu berprfendes Dokument.
	 * @param c Verbindungsdaten.
	 * @return Feld von Dokumenten deren Signatur stimmte.
	 */
	public Document[] verify(Document d, Context c) {
		if (d == null)
			return new Document[0];
		return manager.verifyDocument(d, c);
	}
	/**
	 * Delegiert das Signieren eines Dokuments an den Manager weiter.
	 * @param d Zu signierendes Dokument.
	 * @param c Verbindungsdaten.
	 * @return Signiertes Dokument.
	 */
	public Document sign(Document d, Context c) {
		if (d == null)
			return null;
		return manager.signDocument(d, c);
	}
	/**
	 * Berechnet einen Hashwert fr den String.
	 * @param s Zu hashender String.
	 * @return String mit dem Hashwert.
	 */
	public String hash(String s) {
		String result = null;
		byte[] tmp = s.getBytes();
		tmp = digest.digest(tmp);
		String hash = new String(tmp);
		result = "" + hash.hashCode();
		return result;
	}
	/**
	 * Verarbeitung einer vom Kunden empfangenen Bestellung.
	 * @param request Bestelldokument.
	 * @param context Verbindungsdaten.
	 * @return Dokument mit Mangel-/Fehlerbeschreibung oder die Lieferung.
	 */
	public Document handleRequest(Document request, Context context) {

		Document result = null;

		manager.registerContext(context);
		// Erstmal testen...
		Document[] docs = verify(request, context);
		if ((docs == null) || (docs.length != 1)) {
			if (docs.length == 0) {
				System.out.println(
					"Treuhaender - Signatur der Bestellung "
						+ "ueberprueft und NICHT akzeptiert!");
				System.out.println(
					"Treuhaender - Sende MangelKunde-" + "Dokument.");
				result =
					getKundeMangel(
						context,
						TName_,
						TAdresse_,
						TTelefon_,
						TIPAdresse_,
						TPort_,
						"?",
						"?",
						"?",
						"?",
						"Kunde: Ungueltige Signatur");
			} else {
				System.out.println(
					"Treuhaender - Signatur der Bestellung "
						+ "ueberprueft und akzeptiert!");
				System.out.println(
					"Treuhaender - Signatur der Bestellung "
						+ "verletzt Protokoll.");
				System.out.println(
					"Treuhaender - Sende MangelKunde-" + "Dokument.");
				result =
					getKundeMangel(
						context,
						TName_,
						TAdresse_,
						TTelefon_,
						TIPAdresse_,
						TPort_,
						"?",
						"?",
						"?",
						"?",
						"Kunde: Protokollverletzung");
			}
		} else {
			String KName = "";
			String KIPAdresse = "";
			String KPort = "";
			String HName = "";
			String HIPAdresse = "";
			String HPort = "";
			String Katalog_Nr = "";
			String Ges_Preis = "";
			String Geld = "";
			int Bestell_ID = 0;
			try {
				// Exceptions beenden die Verarbeitung und resultieren in
				// einem KundeMangel-Dokument

				request = docs[0];
				System.out.println(request);
				//Document response = null;
				NodeList Ware = null;
				// Variablen fuer Informationen aus dem Dokument
				// Ausgenommen ist die Gesamtbestellung
				String[] dataname =
					{
						"KIPAdresse",
						"KPort",
						"HName",
						"HIPAdresse",
						"HPort",
						"Katalog_Nr",
						"Ges_Preis",
						"Geld" };
				String[] dataval =
					{
						"kname",
						"kipadresse",
						"kport",
						"hname",
						"hipadresse",
						"hport",
						"katnr",
						"gespreis",
						"geld" };
				// nur schon mal vorinit.

				request.getDocumentElement().normalize();
				//Element rootelement  = request.getDocumentElement();
				NodeList bestelllist =
					request.getElementsByTagName("Bestellungkt");

				// Ist es eine Bestellung?
				if (bestelllist == null) {
					System.out.println(
						"Treuhaender - Es handelt sich nicht "
							+ "um eine Bestellung!");
					result =
						getKundeMangel(
							context,
							TName_,
							TAdresse_,
							TTelefon_,
							TIPAdresse_,
							TPort_,
							"?",
							"?",
							"?",
							"?",
							"Kunde: keine Bestellung");
					throw new Exception();
				}
				System.out.println(
					"Treuhaender - Bestellung empfangen - "
						+ "Start Bestellvorgang");
				// Von wem kommt die Bestellung?
				NodeList datalist = request.getElementsByTagName("KName");
				Node data = datalist.item(0);
				Node dat;
				if (data == null) {
					System.out.println(
						"Treuhaender - Kundeninformationen "
							+ "konnten im Dokument nicht gefunden "
							+ "werden");
					result =
						getKundeMangel(
							context,
							TName_,
							TAdresse_,
							TTelefon_,
							TIPAdresse_,
							TPort_,
							"?",
							"?",
							"?",
							"?",
							"Kunde: keine Kundeninformation");
					throw new Exception();
				}
				dat = data.getFirstChild();
				KName = dat.getNodeValue();
				System.out.println(
					"Treuhaender - Name (Pseudonym) " + "des Kunden: " + KName);

				// Restliche Daten aus Dokument holen
				try {
					for (int i = 0; i < dataname.length; i++) {
						datalist = request.getElementsByTagName(dataname[i]);
						data = datalist.item(0);
						dat = data.getFirstChild();
						dataval[i] = dat.getNodeValue();
					}
				} catch (Exception e) {
					System.out.println(
						"Treuhaender - Es trat ein "
							+ "Fehler beim "
							+ "Auswerten des Dokuments auf");
					e.printStackTrace();
					System.out.println(
						"Treuhaender - Der Vorgang wird " + "abgebrochen!");
					result =
						getKundeMangel(
							context,
							TName_,
							TAdresse_,
							TTelefon_,
							TIPAdresse_,
							TPort_,
							"?",
							"?",
							"?",
							"?",
							"Treuhaender: " + "Bestellverarbeitung fehlerhaft");
					throw new Exception();
				}
				KIPAdresse = dataval[0];
				KPort = dataval[1];
				HName = dataval[2];
				HIPAdresse = dataval[3];
				HPort = dataval[4];
				Katalog_Nr = dataval[5];
				Ges_Preis = dataval[6];
				Geld = dataval[7];

				//Pruefen ob 1. Der Gesamtpreis der Summe der Einzelpreise
				//              entspricht
				//           2. Das Geld dem Gesamtpreis entspricht
				NodeList bestdata = request.getElementsByTagName("Preis");
				Node bedata;
				Node endata;
				try {
					int sumprice = 0;
					for (int i1 = 0; i1 < bestdata.getLength(); i1++) {
						bedata = bestdata.item(i1);
						endata = bedata.getFirstChild();
						Float price =
							new Float(parsePrice(endata.getNodeValue()));
						sumprice += (int) (price.floatValue() * 100);
					}

					int bigprice = 0;
					bestdata = request.getElementsByTagName("Ges_Preis");
					bedata = bestdata.item(0);
					endata = bedata.getFirstChild();
					Float price2 = new Float(parsePrice(endata.getNodeValue()));
					bigprice = (int) (price2.floatValue() * 100);

					// 1.Test entspricht Gesamtpreis der Summe
					// der Einzelpreise?
					if (bigprice != sumprice) {
						//Fehlerfall, Mangelkunde zurueckschicken
						System.out.println(
							"Treuhaender - Gesamtpreis "
								+ "entspricht nicht Summe der "
								+ "Einzelpreise!");
						System.out.println(
							"Treuhaender - Sende " + "MangelKunde-Dokument.");
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Kunde: Gesamtpreis != " + "sum(Einzelpreise)");
					}

					int realmoney = 0;
					Float money = new Float(parsePrice(Geld));
					realmoney = (int) (money.floatValue() * 100);

					// 2.Test entspricht Geld dem Gesamtpreis
					if (bigprice != realmoney) {
						//Fehlerfall, Mangelkunde zurueckschicken
						System.out.println(
							"Treuhaender - Gesamtpreis "
								+ "entspricht nicht "
								+ "dem mitgeliefertem Geld!");
						System.out.println(
							"Treuhaender - Sende " + "MangelKunde-Dokument.");
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Kunde: Gesamtpreis != Geld ");
						throw new Exception();
					}
				} catch (Exception e) {
					System.out.println(
						"Es trat ein Fehler beim Auswerten "
							+ "des Preises auf "
							+ e);
					if (result == null) {
						e.printStackTrace();
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Treuhaender: "
									+ "Fehlerhafte Preisauswertung");

					}
					System.out.println("Der Vorgang wird abgebrochen!");
					throw new Exception();
				}

				// An dieser Stelle stimmen also Signatur, Preise - 
				// Geld und Dokument
				// d.h. die Bestellung wird an den Haendler weitergeleitet

				// Zusammenbau des Bestelldokuments
				// Die BestellID entspricht dem Counter zu diesem Zeitpunkt
				Bestell_ID = counter;
				counter++;
				Document bestelldoc = null;
				System.out.println(
					"Treuhaender - beginne Bestellung an "
						+ "Haendler zu bearbeiten");
				try {
					bestelldoc = db.newDocument();

					// Alle Elemente erzeugen
					Element root =
						(Element) bestelldoc.createElement("Bestellungth");
					Element Treuhaend =
						(Element) bestelldoc.createElement("Treuhaender");
					Element Haendler =
						(Element) bestelldoc.createElement("Haendler");
					Element Katalognr =
						(Element) bestelldoc.createElement("Katalog_Nr");
					Element Bestellid =
						(Element) bestelldoc.createElement("Bestell_ID");
					Element Gesamtpr =
						(Element) bestelldoc.createElement("Ges_Preis");
					Element Gesamtbe =
						(Element) bestelldoc.createElement("Gesamtbestellung");
					Element Tadresse =
						(Element) bestelldoc.createElement("TAdresse");
					Element Tname = (Element) bestelldoc.createElement("TName");
					Element Ttelefon =
						(Element) bestelldoc.createElement("TTelefon");
					Element Tipadresse =
						(Element) bestelldoc.createElement("TIPAdresse");
					Element Tport = (Element) bestelldoc.createElement("TPort");
					Element Hname = (Element) bestelldoc.createElement("HName");
					Element Hipadresse =
						(Element) bestelldoc.createElement("HIPAdresse");
					Element Hport = (Element) bestelldoc.createElement("HPort");

					// Alle Textknoten erzeugen
					Katalognr.appendChild(
						bestelldoc.createTextNode(Katalog_Nr));
					Tname.appendChild(bestelldoc.createTextNode(TName_));
					Ttelefon.appendChild(bestelldoc.createTextNode(TTelefon_));
					Tipadresse.appendChild(
						bestelldoc.createTextNode(TIPAdresse_));
					Tadresse.appendChild(bestelldoc.createTextNode(TAdresse_));
					Tport.appendChild(bestelldoc.createTextNode(TPort_));
					Hname.appendChild(bestelldoc.createTextNode(HName));
					Hipadresse.appendChild(
						bestelldoc.createTextNode(HIPAdresse));
					Hport.appendChild(bestelldoc.createTextNode(HPort));
					Bestellid.appendChild(
						bestelldoc.createTextNode("" + Bestell_ID));
					Gesamtpr.appendChild(bestelldoc.createTextNode(Ges_Preis));

					// Alles Zusammenbauen
					Treuhaend.appendChild(Tname);
					Treuhaend.appendChild(Tadresse);
					Treuhaend.appendChild(Ttelefon);
					Treuhaend.appendChild(Tipadresse);
					Treuhaend.appendChild(Tport);
					Haendler.appendChild(Hname);
					Haendler.appendChild(Hipadresse);
					Haendler.appendChild(Hport);

					NodeList ids = request.getElementsByTagName("ID");
					NodeList preise = request.getElementsByTagName("Preis");
					if (ids.getLength() != preise.getLength()) {
						System.out.println(
							"Treuhaender - Anzahl der "
								+ "Preise != Anzahl der IDs "
								+ "in Bestellung");
						System.out.println(
							"Treuhaender - Vorgang " + "wird abgebrochen!");
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Kunde: Anzahl "
									+ "Preise != Anzahl IDs "
									+ "in Bestellung ");
						throw new Exception();
					}
					hashtable.put(
						new Integer(Bestell_ID),
						new Integer(ids.getLength()));
					for (int i = 0; i < ids.getLength(); i++) {
						Element eintrag =
							(Element) bestelldoc.createElement("Eintrag");
						Element preisel =
							(Element) bestelldoc.createElement("Preis");
						Element ideleme =
							(Element) bestelldoc.createElement("ID");
						Node preis = preise.item(i);
						Node id = ids.item(i);
						preisel.appendChild(
							bestelldoc.createTextNode(
								preis.getFirstChild().getNodeValue()));
						ideleme.appendChild(
							bestelldoc.createTextNode(
								id.getFirstChild().getNodeValue()));
						eintrag.appendChild(ideleme);
						eintrag.appendChild(preisel);
						Gesamtbe.appendChild(eintrag);
					}
					root.appendChild(Treuhaend);
					root.appendChild(Haendler);
					root.appendChild(Katalognr);
					root.appendChild(Bestellid);
					root.appendChild(Gesamtbe);
					root.appendChild(Gesamtpr);
					bestelldoc.appendChild(root);
				} catch (Exception e) {
					System.out.println(
						"Treuhaender - Fehler beim "
							+ "Generieren des "
							+ "Bestellungth-Dokuments! "
							+ e);
					if (result == null) {
						e.printStackTrace();
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Treuhaender: "
									+ "Fehler in "
									+ "Bestellungsgenerierung");
					}
					throw new Exception();
				}
				// Signieren des Dokuments
				Document bestelldocsig = sign(bestelldoc, context);
				System.out.println(
					"Treuhaender - signiere die Bestellung "
						+ "an den Haendler");
				// Senden des Dokuments
				counter++; // Zaehler erhoehen

				// Antwort vom Haendler holen
				Document answer =
					sendTo(
						context,
						HIPAdresse,
						HPort,
						bestelldocsig,
						Ges_Preis,
						"Bestellung");
				// Wenn Anwort null - Fehler
				// Was mit Kunden machen, wenn Haendler Fehler macht?
				if (answer == null) {
					result =
						getKundeMangel(
							context,
							TName_,
							TAdresse_,
							TTelefon_,
							TIPAdresse_,
							TPort_,
							KName,
							KIPAdresse,
							KPort,
							Geld,
							"Treuhaender: Fehler bei " + "Haendlerkontakt");
					throw new Exception();
				}
				Ware = answer.getElementsByTagName("Ware");

				// Haendler bezahlen
				// Zusammenbau des Bezahlungsdokuments
				Document bezahlungsdoc = null;
				System.out.println(
					"Treuhaender - beginne Bezahldokument "
						+ "fuer Haendler zu bearbeiten");
				try {
					bezahlungsdoc = db.newDocument();

					// Alle Elemente erzeugen
					Element root =
						(Element) bezahlungsdoc.createElement("Bezahlung");
					Element Treuhaend =
						(Element) bezahlungsdoc.createElement("Treuhaender");
					Element Haendler =
						(Element) bezahlungsdoc.createElement("Haendler");
					Element Bestellid =
						(Element) bezahlungsdoc.createElement("Bestell_ID");
					Element Gesamtpr =
						(Element) bezahlungsdoc.createElement("Ges_Preis");
					Element Geldb =
						(Element) bezahlungsdoc.createElement("Geld");
					Element Tadresse =
						(Element) bezahlungsdoc.createElement("TAdresse");
					Element Tname =
						(Element) bezahlungsdoc.createElement("TName");
					Element Ttelefon =
						(Element) bezahlungsdoc.createElement("TTelefon");
					Element Tipadresse =
						(Element) bezahlungsdoc.createElement("TIPAdresse");
					Element Tport =
						(Element) bezahlungsdoc.createElement("TPort");
					Element Hname =
						(Element) bezahlungsdoc.createElement("HName");
					Element Hipadresse =
						(Element) bezahlungsdoc.createElement("HIPAdresse");
					Element Hport =
						(Element) bezahlungsdoc.createElement("HPort");

					// Alle Textknoten erzeugen
					Tname.appendChild(bezahlungsdoc.createTextNode(TName_));
					Ttelefon.appendChild(
						bezahlungsdoc.createTextNode(TTelefon_));
					Tipadresse.appendChild(
						bezahlungsdoc.createTextNode(TIPAdresse_));
					Tadresse.appendChild(
						bezahlungsdoc.createTextNode(TAdresse_));
					Tport.appendChild(bezahlungsdoc.createTextNode(TPort_));
					Hname.appendChild(bezahlungsdoc.createTextNode(HName));
					Hipadresse.appendChild(
						bezahlungsdoc.createTextNode(HIPAdresse));
					Hport.appendChild(bezahlungsdoc.createTextNode(HPort));
					Bestellid.appendChild(
						bezahlungsdoc.createTextNode("" + Bestell_ID));
					Gesamtpr.appendChild(
						bezahlungsdoc.createTextNode(Ges_Preis));
					Geldb.appendChild(bezahlungsdoc.createTextNode(Ges_Preis));

					// Alles Zusammenbauen
					Treuhaend.appendChild(Tname);
					Treuhaend.appendChild(Tadresse);
					Treuhaend.appendChild(Ttelefon);
					Treuhaend.appendChild(Tipadresse);
					Treuhaend.appendChild(Tport);
					Haendler.appendChild(Hname);
					Haendler.appendChild(Hipadresse);
					Haendler.appendChild(Hport);

					root.appendChild(Treuhaend);
					root.appendChild(Haendler);
					root.appendChild(Bestellid);
					root.appendChild(Gesamtpr);
					root.appendChild(Geldb);
					bezahlungsdoc.appendChild(root);

				} catch (Exception e) {
					System.out.println(
						"Treuhaender - Fehler beim Generieren "
							+ "des Bezahlungs-Dokuments! "
							+ e);
					if (result == null) {
						e.printStackTrace();
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Treuhaender: Fehler bei "
									+ "Bezahlungsgenerierung");
					}
					throw new Exception();
				}
				// Signieren des Dokuments
				System.out.println(
					"Treuhaender - signierte das "
						+ "Bezahlungsdokument an den Haendler");
				Document bezahlungsdocsig = sign(bezahlungsdoc, context);
				// Senden des Dokuments
				System.out.println(
					"Treuhaender - sende Bezahlung " + "zum Haendler");
				Document bresponse =
					sendTo(
						context,
						HIPAdresse,
						HPort,
						bezahlungsdocsig,
						Ges_Preis,
						"Bezahlung");
				// Antwort auf Bezahlung muss null sein
				if (bresponse != null) {
					System.out.println(
						"Fehler, nach Bezahlung darf "
							+ "keine Antwort erfolgen!");
					System.out.println("Ignoriert!");
				}

				// Ware an Kunden versenden
				// Zusammenbau des Bezahlungsdokuments
				Document lieferungtkdoc = null;
				System.out.println(
					"Treuhaender - beginne Lieferung fuer "
						+ "Kunden zu bearbeiten");
				try {
					lieferungtkdoc = db.newDocument();

					// Alle Elemente erzeugen
					Element root =
						(Element) lieferungtkdoc.createElement("Lieferungtk");
					Element Treuhaend =
						(Element) lieferungtkdoc.createElement("Treuhaender");
					Element Kunde =
						(Element) lieferungtkdoc.createElement("Kunde");
					Element Tadresse =
						(Element) lieferungtkdoc.createElement("TAdresse");
					Element Tname =
						(Element) lieferungtkdoc.createElement("TName");
					Element Ttelefon =
						(Element) lieferungtkdoc.createElement("TTelefon");
					Element Tipadresse =
						(Element) lieferungtkdoc.createElement("TIPAdresse");
					Element Tport =
						(Element) lieferungtkdoc.createElement("TPort");
					Element Kname =
						(Element) lieferungtkdoc.createElement("KName");
					Element Kipadresse =
						(Element) lieferungtkdoc.createElement("KIPAdresse");
					Element Kport =
						(Element) lieferungtkdoc.createElement("KPort");

					// Alle Textknoten erzeugen
					Tname.appendChild(lieferungtkdoc.createTextNode(TName_));
					Ttelefon.appendChild(
						lieferungtkdoc.createTextNode(TTelefon_));
					Tipadresse.appendChild(
						lieferungtkdoc.createTextNode(TIPAdresse_));
					Tadresse.appendChild(
						lieferungtkdoc.createTextNode(TAdresse_));
					Tport.appendChild(lieferungtkdoc.createTextNode(TPort_));
					Kname.appendChild(lieferungtkdoc.createTextNode(KName));
					Kipadresse.appendChild(
						lieferungtkdoc.createTextNode(KIPAdresse));
					Kport.appendChild(lieferungtkdoc.createTextNode(KPort));

					// Alles Zusammenbauen
					Treuhaend.appendChild(Tname);
					Treuhaend.appendChild(Tadresse);
					Treuhaend.appendChild(Ttelefon);
					Treuhaend.appendChild(Tipadresse);
					Treuhaend.appendChild(Tport);
					Kunde.appendChild(Kname);
					Kunde.appendChild(Kipadresse);
					Kunde.appendChild(Kport);

					root.appendChild(Treuhaend);
					root.appendChild(Kunde);

					for (int i = 0; i < Ware.getLength(); i++) {
						Element waree =
							(Element) lieferungtkdoc.createElement("Ware");
						Element book =
							(Element) lieferungtkdoc.createElement("Book");
						Element hash =
							(Element) lieferungtkdoc.createElement(
								"BSignatur_H");

						Node warnod = Ware.item(i);
						Node warchild = warnod.getFirstChild();
						while (warchild != null) {
							if (warchild.getNodeName().equals("Book")) {
								book.appendChild(
									lieferungtkdoc.createTextNode(
										warchild
											.getFirstChild()
											.getNodeValue()));
							}
							if (warchild.getNodeName().equals("BSignatur_H")) {
								hash.appendChild(
									lieferungtkdoc.createTextNode(
										warchild
											.getFirstChild()
											.getNodeValue()));
							}

							waree.appendChild(book);
							waree.appendChild(hash);
							root.appendChild(waree);
							warchild = warchild.getNextSibling();
						}
					}
					lieferungtkdoc.appendChild(root);
				} catch (Exception e) {
					System.out.println(
						"Treuhaender - Fehler beim Generieren "
							+ "des Lieferungs-Dokuments! "
							+ e);
					if (result == null) {
						e.printStackTrace();
						result =
							getKundeMangel(
								context,
								TName_,
								TAdresse_,
								TTelefon_,
								TIPAdresse_,
								TPort_,
								KName,
								KIPAdresse,
								KPort,
								Geld,
								"Treuhaender: Fehler bei "
									+ "Bezahlungsgenerierung");
					}
					throw new Exception();
				}
				// Signieren des Dokuments
				System.out.println(
					"Treuhaender - signiere das "
						+ "Lieferungssdokument fuer den Kunden");
				Document lieferungtkdocsig = sign(lieferungtkdoc, context);
				System.out.println("Treuhaender - sende Lieferung zum Kunden");
				System.out.println("Treuhaender - Vorgang abgeschlossen");

				result = lieferungtkdocsig;
			} catch (Exception e) {
				System.out.println("Treuhaender - Fehler in handleRequest.");
				if (result == null) {
					e.printStackTrace();
					result =
						getKundeMangel(
							context,
							TName_,
							TAdresse_,
							TTelefon_,
							TIPAdresse_,
							TPort_,
							KName,
							KIPAdresse,
							KPort,
							Geld,
							"Treuhaender: Fehler bei " + "Anfrageverarbeitung");
				}
			}
		}
		manager.unregisterContext(context);
		return result;
	}
	/**
	 * Whlt vokonfigurierte Rolle aus.
	 * @param host Wird ignoriert.
	 * @param port Wird ignoriert.
	 * @return <b>Treuhnder</b> 
	 */
	protected String chooseRole(String host, int port) {
		return "Treuhnder";
	}

	/**
	 *  Sendet ein Dokument zu einem Haendler und wertet die Antwort aus.
	 *  Stimmt das Dokument im allgemeinen, die Signatur und die Ware, wird das
	 *  Dokument zurueckgeliefert.
	 *
	 *  @param String host, String port, File request, String Ges_Preis
	 *  @return Document okresponse
	 */
	private Document sendTo(
		Context context_,
		String host,
		String sport,
		Document request,
		String Ges_Preis,
		String action) {
		String Begruendung;
		System.out.println(
			"Treuhaender - sende signiertes Dokument an " + "den Haendler");
		// CONTEXT ERSTELLEN
		Document signedDocument = request;
		try {
			String hostname = host;
			int port = new Integer(sport).intValue();
			String rolename = chooseRole(host, port);
			Context context = createContext(rolename, action);
			InetAddress target = null;
			try {
				target = InetAddress.getByName(hostname);
			} catch (UnknownHostException uhe) {
			}
			// TODO: Fehlerbehandlung!
			if (target == null) {
			}
			// Fuellen
			context.setTargetAddress(target);
			context.setTargetPort(port);
			// VERBINDUNG HERSTELLEN
			TrusteeToSalesMan cc = new TrusteeToSalesMan(context);
			// SSONET HANDSHAKE
			boolean ok = cc.connect();
			// TODO: Fehlerbehandlung!
			if (!ok) {
			}
			manager.registerContext(context);
			// DOKUMENT SENDEN
			signedDocument = cc.perform(signedDocument);
			System.out.println(
				"Bestellung abgeschickt -> Zeit : " + new Date());
			// TODO: Fehlerbehandlung!
			if (signedDocument == null) {
				//Bei Bezahlung oki
				manager.unregisterContext(context);
				return null;
			}
			// SIGNATUR PRUEFEN
			Document[] referenced = verify(signedDocument, context);
			// TODO: Fehlerbehandlung!
			if (referenced == null || referenced.length != 1) {
				System.out.println("Haendlersignatur ungueltig...");
				manager.unregisterContext(context);
				return null;
			}
			// AUFRAEUMEN
			signedDocument = referenced[0];
			cc.disconnect();
			manager.unregisterContext(context);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		Document hresponse = signedDocument;
		Document lieferunght = hresponse;

		//Element rootlieferht = hresponse.getDocumentElement();
		NodeList lieferlist = hresponse.getElementsByTagName("Lieferunght");

		// Ist es eine Lieferung?
		if (lieferlist != null) {
			System.out.println(
				"Treuhaender - Lieferung empfangen - Start Bearbeitung");
		} else {
			System.out.println(
				"Treuhaender - Es handelt sich nicht um eine Lieferung!");
			return null;
		}

		// Daten aus Dokument holen
		NodeList datalistl = lieferunght.getElementsByTagName("HName");
		String HNamel = datalistl.item(0).getFirstChild().getNodeValue();
		datalistl = lieferunght.getElementsByTagName("HIPAdresse");
		String HIPAdressel = datalistl.item(0).getFirstChild().getNodeValue();
		datalistl = lieferunght.getElementsByTagName("HPort");
		String HPortl = datalistl.item(0).getFirstChild().getNodeValue();

		datalistl = lieferunght.getElementsByTagName("Bestell_ID");
		String Bestell_IDl = datalistl.item(0).getFirstChild().getNodeValue();
		NodeList Warel = lieferunght.getElementsByTagName("Ware");

		//System.out.println("Treuhaender " + TName + "  " + "Haendler " + HNamel);
		System.out.println(
			"Treuhaender - Signatur der Lieferung ueberprueft und akzeptiert");

		// Test ob Anzahl der Bestellten Ware = Anzahl der gelieferten Buechen
		if (Warel.getLength()
			!= ((Integer) hashtable.get(new Integer(Bestell_IDl))).intValue()) {
			System.out.println("Anzahl Ware: " + Warel.getLength());
			System.out.println(
				"Anzahl in Bestellung: "
					+ ((Integer) hashtable.get(new Integer(Bestell_IDl)))
						.intValue());
			System.out.println(
				"Treuhaender - Anzahl der Waren != Anzahl der bestellten Waren!");
			System.out.println("Treuhaender - Sende MangelHaendler-Dokument.");
			Begruendung =
				"Treuhaender: Anzahl der Waren != "
					+ "Anzahl der bestellten Waren";
			lieferunght =
				getHaendlerMangel(
					context_,
					TName_,
					TAdresse_,
					TTelefon_,
					TIPAdresse_,
					TPort_,
					HNamel,
					HIPAdressel,
					HPortl,
					Bestell_IDl,
					Ges_Preis,
					Begruendung,
					Warel);
			lieferunght =
				sendTo(context_, host, sport, lieferunght, Ges_Preis, "Mangel");
			return lieferunght;
		}

		// Ist die Ware i.O.
		// Alle Posten durchgehen und Hashwert vergleichen
		boolean wareok = true;
		System.out.println("Treuhaender - teste Ware des Haendlers");
		for (int i = 0; i < Warel.getLength(); i++) {
			String book = "";
			String hash = "";
			Node einzelware = Warel.item(i);
			Node warchild = einzelware.getFirstChild();
			while (warchild != null) {
				if (warchild.getNodeName().equals("Book")) {
					book = warchild.getFirstChild().getNodeValue();
				}
				if (warchild.getNodeName().equals("BSignatur_H")) {
					hash = warchild.getFirstChild().getNodeValue();
				}
				warchild = warchild.getNextSibling();
			}
			/*
			System.out.println("Book: " + book + " Hash: " + hash + 
				       " MyHash: " + sig.getHash(book));
			*/
			if (!hash.equals(hash(book)))
				wareok = false;
		}

		if (!wareok) {
			System.out.println("Treuhaender - Fehlerhafte Ware!");
			System.out.println("Treuhaender - Sende MangelHaendler-Dokument.");
			Begruendung = "Treuhaender: Fehlerhafte Ware";
			lieferunght =
				getHaendlerMangel(
					context_,
					TName_,
					TAdresse_,
					TTelefon_,
					TIPAdresse_,
					TPort_,
					HNamel,
					HIPAdressel,
					HPortl,
					Bestell_IDl,
					Ges_Preis,
					Begruendung,
					Warel);
			return sendTo(
				context_,
				host,
				sport,
				lieferunght,
				Ges_Preis,
				"Mangel");
		} else {
			System.out.println("Treuhaender - Ware ist in Ordnung");
		}

		// Falls bis hierher durchgelaufen scheint alles ok zu sein, also Document zurueckliefern
		return lieferunght;
	}

	/**
	 *  Gibt das Mangeldokument an den Haendler zurueck
	 *
	 *  @param Context context, String TName, String TAdresse, String TTelefon, String TIPAdresse, String TPort, String HName, String HIPAdresse, String HPort, String BestellID, String Ges_Preis, String Reklamationsbegruendung, NodeList Ware
	 *  @return Document Mangeldokument
	 */
	private Document getHaendlerMangel(
		Context context,
		String tnames,
		String tadresses,
		String ttelefons,
		String tipadresses,
		String tports,
		String hnames,
		String hipadresses,
		String hports,
		String bestellid,
		String gespreis,
		String reklbegruendung,
		NodeList ware) {
		Document doc = null;

		try {
			doc = db.newDocument();

			// Alle Elemente erzeugen
			Element Mhaendler = (Element) doc.createElement("Mangelhaendler");
			Element Treuhaend = (Element) doc.createElement("Treuhaender");
			Element Haendler = (Element) doc.createElement("Haendler");
			Element Reklamat = (Element) doc.createElement("Reklamation");
			Element Bestellid = (Element) doc.createElement("Bestell_ID");
			Element Gesamtpr = (Element) doc.createElement("Ges_Preis");
			Element Tadresse = (Element) doc.createElement("TAdresse");
			Element Tname = (Element) doc.createElement("TName");
			Element Ttelefon = (Element) doc.createElement("TTelefon");
			Element Tipadresse = (Element) doc.createElement("TIPAdresse");
			Element Tport = (Element) doc.createElement("TPort");
			Element Hname = (Element) doc.createElement("HName");
			Element Hipadresse = (Element) doc.createElement("HIPAdresse");
			Element Hport = (Element) doc.createElement("HPort");
			Element Begruend = (Element) doc.createElement("Begruendung");

			// Alle Textknoten erzeugen
			Tname.appendChild(doc.createTextNode(tnames));
			Ttelefon.appendChild(doc.createTextNode(ttelefons));
			Tipadresse.appendChild(doc.createTextNode(tipadresses));
			Tadresse.appendChild(doc.createTextNode(tadresses));
			Tport.appendChild(doc.createTextNode(tports));
			Hname.appendChild(doc.createTextNode(hnames));
			Hipadresse.appendChild(doc.createTextNode(hipadresses));
			Hport.appendChild(doc.createTextNode(hports));
			Bestellid.appendChild(doc.createTextNode(bestellid));
			Gesamtpr.appendChild(doc.createTextNode(gespreis));
			Begruend.appendChild(doc.createTextNode(reklbegruendung));

			// Alles noch schoen zusammensetzen
			Reklamat.appendChild(Begruend);
			for (int i = 0; i < ware.getLength(); i++) {
				Element waree = (Element) doc.createElement("Ware");
				Element book = (Element) doc.createElement("Book");
				Element hash = (Element) doc.createElement("BSignatur_H");

				Node warnod = ware.item(i);
				Node warchild = warnod.getFirstChild();
				while (warchild != null) {
					if (warchild.getNodeName().equals("Book")) {
						book.appendChild(
							doc.createTextNode(
								warchild.getFirstChild().getNodeValue()));
					}
					if (warchild.getNodeName().equals("BSignatur_H")) {
						hash.appendChild(
							doc.createTextNode(
								warchild.getFirstChild().getNodeValue()));
					}

					waree.appendChild(book);
					waree.appendChild(hash);
					Reklamat.appendChild(waree);

					warchild = warchild.getNextSibling();
				}
			}

			Treuhaend.appendChild(Tname);
			Treuhaend.appendChild(Tadresse);
			Treuhaend.appendChild(Ttelefon);
			Treuhaend.appendChild(Tipadresse);
			Treuhaend.appendChild(Tport);
			Haendler.appendChild(Hname);
			Haendler.appendChild(Hipadresse);
			Haendler.appendChild(Hport);

			Mhaendler.appendChild(Treuhaend);
			Mhaendler.appendChild(Haendler);
			Mhaendler.appendChild(Reklamat);
			Mhaendler.appendChild(Bestellid);
			Mhaendler.appendChild(Gesamtpr);
			doc.appendChild(Mhaendler);

		} catch (Exception e) {
			System.out.println(
				"Fehler beim Generieren des HaendlerMangel-Dokuments! " + e);
		}

		// Am Ende natuerlich noch signieren
		doc = sign(doc, context);
		return doc;
	}

	/**
	 *  Gibt das Mangeldokument fuer den Kunden zurueck
	 *
	 *  @param Context context, String TName, String TAdresse, String TTelefon, String TIPAdresse, String TPort, String KName, String KIPAdresse, String KPort, String Geld
	 *  @return Document Mangeldokument
	 */
	private Document getKundeMangel(
		Context context,
		String tnames,
		String tadresses,
		String ttelefons,
		String tipadresses,
		String tports,
		String knames,
		String kipadresses,
		String kports,
		String gelds,
		String status) {
		Document doc = null;

		try {
			doc = db.newDocument();

			// Alle Elemente erzeugen
			Element Mkunde = (Element) doc.createElement("Mangelkunde");
			Element Kunde = (Element) doc.createElement("Kunde");
			Element Treu = (Element) doc.createElement("Treuhaender");
			Element geld = (Element) doc.createElement("Geld");
			Element stat = (Element) doc.createElement("Status");
			Element tname = (Element) doc.createElement("TName");
			Element tadresse = (Element) doc.createElement("TAdresse");
			Element ttelefon = (Element) doc.createElement("TTelefon");
			Element tipadresse = (Element) doc.createElement("TIPAdresse");
			Element tport = (Element) doc.createElement("TPort");
			Element kname = (Element) doc.createElement("KName");
			Element kipadresse = (Element) doc.createElement("KIPAdresse");
			Element kport = (Element) doc.createElement("KPort");

			// Alle Textknoten erzeugen
			stat.appendChild(doc.createTextNode(status));
			tname.appendChild(doc.createTextNode(tnames));
			tadresse.appendChild(doc.createTextNode(tadresses));
			ttelefon.appendChild(doc.createTextNode(ttelefons));
			tipadresse.appendChild(doc.createTextNode(tipadresses));
			tport.appendChild(doc.createTextNode(tports));
			kname.appendChild(doc.createTextNode(knames));
			kipadresse.appendChild(doc.createTextNode(kipadresses));
			kport.appendChild(doc.createTextNode(kports));
			geld.appendChild(doc.createTextNode(gelds));

			// Jetzt noch alles schoen zusammensetzen
			Kunde.appendChild(kname);
			Kunde.appendChild(kipadresse);
			Kunde.appendChild(kport);
			Treu.appendChild(tname);
			Treu.appendChild(tadresse);
			Treu.appendChild(ttelefon);
			Treu.appendChild(tipadresse);
			Treu.appendChild(tport);

			Mkunde.appendChild(Treu);
			Mkunde.appendChild(Kunde);
			Mkunde.appendChild(stat);
			Mkunde.appendChild(geld);

			doc.appendChild(Mkunde);

		} catch (Exception e) {
			System.out.println(
				"Fehler beim Generieren des MangelKunde-Dokuments! " + e);
		}
		// Am Ende natuerlich noch signieren
		doc = sign(doc, context);
		return doc;
	}

	/**
	 *  Parst den Geldstring. Mit "." statt ",".
	 *  @param price Alter Preis.
	 *  @return String mit korrekt dargestelltem float-Wert.
	 */
	protected String parsePrice(String price) {
		if (price == null)
			return "666";
		price = price.replace(',', '.');
		StringTokenizer st = new StringTokenizer(price, "DM");
		price = "";
		while (st.hasMoreTokens())
			price += st.nextToken();
		return price;
	}
}
