/**
 * @version working
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * $Id: TrusteeToSalesMan.java,v 1.9 2004/02/12 08:12:41 tk4 Exp $
 */
package treuhaender;

import idman.mngt.Context;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ssonet.net.ActionConfiguration;
import org.ssonet.net.ConnectionListener;
import org.ssonet.net.SSONETContext;
import org.ssonet.net.impl.SSONETClientSocket;
import org.w3c.dom.Document;

/**
 * Diese Klasse stellt auf Seite des Treuhaenders eine Verbindung mit
 * einem Partner (Haendler) her.
 * @author lehrprojekt
 * @author Thomas Kriegelstein
 * @version $Id: TrusteeToSalesMan.java,v 1.9 2004/02/12 08:12:41 tk4 Exp $
 */
public class TrusteeToSalesMan implements ConnectionListener {

	private Socket socket;
	private InputStream is;
	private OutputStream os;
	private Context context;

	private int port;
	private InetAddress address;

	private static DocumentBuilderFactory dbf = null;
	private static DocumentBuilder db = null;
	private static TransformerFactory tf = null;
	private static Transformer t = null;

	/** Initialisiert die XML-Verarbeitung. */
	static {
		try {
			dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			db = dbf.newDocumentBuilder();
			tf = TransformerFactory.newInstance();
			t = tf.newTransformer();
		} catch (ParserConfigurationException pce) {
			System.err.println(
				"TrusteeToSalesMan<init>: Could not create "
					+ "Factory"
					+ pce.getMessage());
			pce.printStackTrace();
		} catch (TransformerConfigurationException tce) {
			System.err.println(
				"TrusteeToSalesMan<init>: Could not create "
					+ "Factory"
					+ tce.getMessage());
			tce.printStackTrace();
		} /*catch(TransformerException te) {
						           System.err.println("TrusteeToSalesMan<init>: Could not create "+
						           "Transformer" +te.getMessage());
						           te.printStackTrace();
						       }*/
	}
	public synchronized void connectionStateChanged(
		SSONETContext context,
		int currentState) {
		if (currentState == RUNNING_STATE)
			notifyAll();
	}
	public synchronized void waitRunning(SSONETContext context) {
		context.addConnectionListener(this);
		while (context.getConnectionState() != RUNNING_STATE) {
			// wait for notify in connectionStateChanged
			try {
				wait();
			} catch (InterruptedException ie) {
			}
			// allow connectionStateChanged to finish
			// avoids ConcurrentModificationException in the contexts
			//   listeners-list, if no other ConnectionListener is registered
			Thread.yield();
		}
		context.removeConnectionListener(this);
	}

	/**
	 * Baut die Verbindung auf.
	 * @return <b>true</b> fr Erfolg und
	 *         <b>false</b>, falls ein Fehler auftrat.
	 */
	public boolean connect() {
		try {
			socket =
				new SSONETClientSocket(address.getHostName(), port, context);
			System.out.println(
				"TrusteeToSalesman#connect: " + socket.toString());
			waitRunning(context);
			os = socket.getOutputStream();
			is = socket.getInputStream();
			return true;
		} catch (IOException ioe) {
			System.out.println(
				"TrusteeToSalesMan#connect: " + ioe.getMessage());
			try {
				socket.close();
			} catch (Exception e) {
			}
			return false;
		}
	}
	/**
	 * Baut die Verbindung ab.
	 */
	public void disconnect() {
		try {
			is.close();
			os.close();
			socket.close();
		} catch (IOException ioe) {
			System.out.println(
				"TrusteeToSalesMan#disconnect: " + ioe.getMessage());
			try {
				socket.close();
			} catch (Exception e) {
			}
		}
	}
	/**
	 * Konstruktor.
	 * @param context Parametersatz fr den die Verbindung aufgebaut
	 *                werden soll.
	 */
	public TrusteeToSalesMan(Context context) {
		this.context = context;
		this.address = context.getTargetAddress();
		this.port = context.getTargetPort();
		System.out.println("TrusteeToSalesMan:<init>: " + address + ":" + port);
	}
	/**
	 * Arbeitsmethode, bertrgt das Dokument und liefert das Resultat.
	 * @param order Das zu bermittelnde Dokument.
	 * @return <b>org.w3c.Document</b> oder <b>null</b> - das Resultat.
	 */
	public Document perform(Document order) {
		sendRequest(order);
		Document response = getResponse();
		return response;
	}
	private void sendRequest(Document request) {
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			Transformer t = tf.newTransformer();
			t.transform(new DOMSource(request), new StreamResult(baos));

			byte[] tmp = baos.toByteArray();
			baos.close();

			byte[] len =
				{
					(byte) (tmp.length >> 24 & 255),
					(byte) (tmp.length >> 16 & 255),
					(byte) (tmp.length >> 8 & 255),
					(byte) (tmp.length & 255)};
			os.write(len);
			os.write(tmp);
			os.flush();
		} catch (IOException ioe) {
			System.out.println(
				"TrusteeToSalesMan#sendRequest: " + ioe.getMessage());
		} catch (TransformerException te) {
			System.out.println(
				"TrusteeToSalesMan#sendRequest: " + te.getMessage());
		}
	}
	private Document getResponse() {
		try {
			int i, len = 0;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			i = is.read();
			if (i == -1)
				return null;
			else
				len = (len << 8) + i;
			if (len == 0)
				return null;
			System.out.println(len);

			byte[] tmp = new byte[len];
			for (int c = 0, j = 0;
				(c < len) && (-1 != (j = is.read(tmp, c, len - c)));
				c += j);
			ByteArrayInputStream bais = new ByteArrayInputStream(tmp);
			Document response = db.parse(bais);
			return response;
		} catch (Exception e) {
			System.out.println(
				"TrusteeToSalesMan#getResponse: " + e.getMessage());
			e.printStackTrace();
			return null;
		}
	}
	/* (non-Javadoc)
	 * @see org.ssonet.net.ConnectionListener#negotiationRequestReceived(org.ssonet.net.SSONETContext, int)
	 */
	public void negotiationRequestReceived(SSONETContext context, int negotiationType) {
	}
	/* (non-Javadoc)
	 * @see org.ssonet.net.ConnectionListener#decideSecurityGoal(int, org.ssonet.net.ActionConfiguration, org.ssonet.net.SSONETContext)
	 */
	public boolean decideSecurityGoal(int securityGoal, ActionConfiguration ownConfiguration, SSONETContext context) {
		return false;
	}
}
